@@ -23,6 +23,7 @@ import java.math.BigInteger;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
 import java.util.NoSuchElementException;
@@ -35,6 +36,7 @@ import org.apache.commons.configuration.event.ConfigurationErrorListener;
 import org.apache.commons.configuration.event.EventSource;
 import org.apache.commons.configuration.interpol.ConfigurationInterpolator;
 import org.apache.commons.lang.BooleanUtils;
+import org.apache.commons.lang.ClassUtils;
 import org.apache.commons.lang.text.StrLookup;
 import org.apache.commons.lang.text.StrSubstitutor;
 import org.apache.commons.logging.Log;
@@ -76,7 +78,7 @@ import org.apache.commons.logging.impl.NoOpLog;
  * @author <a href="mailto:ksh@scand.com">Konstantin Shaposhnikov </a>
  * @author Oliver Heger
  * @author <a href="mailto:hps@intermeta.de">Henning P. Schmiedehausen </a>
- * @version $Id: AbstractConfiguration.java 652316 2008-04-30 10:59:58Z ebourg $
+ * @version $Id: AbstractConfiguration.java 1153984 2011-08-04 19:57:03Z oheger $
  */
 public abstract class AbstractConfiguration extends EventSource implements Configuration
 {
@@ -558,6 +560,13 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         fireEvent(EVENT_CLEAR, null, null, false);
     }
 
+    /**
+     * {@inheritDoc} This implementation returns keys that either match the
+     * prefix or start with the prefix followed by a dot ('.'). So the call
+     * <code>getKeys("db");</code> will find the keys <code>db</code>,
+     * <code>db.user</code>, or <code>db.password</code>, but not the key
+     * <code>dbdriver</code>.
+     */
     public Iterator getKeys(final String prefix)
     {
         return new FilterIterator(getKeys(), new Predicate()
@@ -1004,7 +1013,7 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
             }
             catch (ConversionException e)
             {
-                throw new ConversionException('\'' + key + "' doesn't map to a BigDecimal object", e);
+                throw new ConversionException('\'' + key + "' doesn't map to a BigInteger object", e);
             }
         }
     }
@@ -1093,6 +1102,11 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         {
             array = new String[0];
         }
+        else if (isScalarValue(value))
+        {
+            array = new String[1];
+            array[0] = value.toString();
+        }
         else
         {
             throw new ConversionException('\'' + key + "' doesn't map to a String/List object");
@@ -1139,6 +1153,10 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         {
             return Arrays.asList((Object[]) value);
         }
+        else if (isScalarValue(value))
+        {
+            return Collections.singletonList(value.toString());
+        }
         else
         {
             throw new ConversionException('\'' + key + "' doesn't map to a List object: " + value + ", a "
@@ -1174,6 +1192,24 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         return value;
     }
 
+    /**
+     * Checks whether the specified object is a scalar value. This method is
+     * called by <code>getList()</code> and <code>getStringArray()</code> if the
+     * property requested is not a string, a list, or an array. If it returns
+     * <b>true</b>, the calling method transforms the value to a string and
+     * returns a list or an array with this single element. This implementation
+     * returns <b>true</b> if the value is of a wrapper type for a primitive
+     * type.
+     *
+     * @param value the value to be checked
+     * @return a flag whether the value is a scalar
+     * @since 1.7
+     */
+    protected boolean isScalarValue(Object value)
+    {
+        return ClassUtils.wrapperToPrimitive(value.getClass()) != null;
+    }
+
     /**
      * Copies the content of the specified configuration into this
      * configuration. If the specified configuration contains a key that is also
