@@ -18,7 +18,6 @@
 package org.apache.commons.configuration;
 
 import java.io.File;
-import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
@@ -27,10 +26,7 @@ import java.io.OutputStreamWriter;
 import java.io.Reader;
 import java.io.UnsupportedEncodingException;
 import java.io.Writer;
-import java.net.HttpURLConnection;
-import java.net.MalformedURLException;
 import java.net.URL;
-import java.net.URLConnection;
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
@@ -78,14 +74,22 @@ import org.apache.commons.logging.LogFactory;
  * in <code>java.util.Properties</code>.</p>
  *
  * @author Emmanuel Bourg
- * @version $Revision: 712401 $, $Date: 2008-11-08 16:29:56 +0100 (Sa, 08 Nov 2008) $
+ * @version $Id: AbstractFileConfiguration.java 1158113 2011-08-16 05:59:54Z oheger $
  * @since 1.0-rc2
  */
-public abstract class AbstractFileConfiguration extends BaseConfiguration implements FileConfiguration
+public abstract class AbstractFileConfiguration
+extends BaseConfiguration
+implements FileConfiguration, FileSystemBased
 {
     /** Constant for the configuration reload event.*/
     public static final int EVENT_RELOAD = 20;
 
+    /** Constant fro the configuration changed event. */
+    public static final int EVENT_CONFIG_CHANGED = 21;
+
+    /** The root of the file scheme */
+    private static final String FILE_SCHEME = "file:";
+
     /** Stores the file name.*/
     protected String fileName;
 
@@ -99,7 +103,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
     protected ReloadingStrategy strategy;
 
     /** A lock object for protecting reload operations.*/
-    private Object reloadLock = new Object();
+    protected Object reloadLock = new Lock("AbstractFileConfiguration");
 
     /** Stores the encoding of the configuration file.*/
     private String encoding;
@@ -110,6 +114,9 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
     /** A counter that prohibits reloading.*/
     private int noReload;
 
+    /** The FileSystem being used for this Configuration */
+    private FileSystem fileSystem = FileSystem.getDefaultFileSystem();
+
     /**
      * Default constructor
      *
@@ -181,6 +188,31 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
         load();
     }
 
+    public void setFileSystem(FileSystem fileSystem)
+    {
+        if (fileSystem == null)
+        {
+            throw new NullPointerException("A valid FileSystem must be specified");
+        }
+        this.fileSystem = fileSystem;
+    }
+
+    public void resetFileSystem()
+    {
+        this.fileSystem = FileSystem.getDefaultFileSystem();
+    }
+
+    public FileSystem getFileSystem()
+    {
+        return this.fileSystem;
+    }
+
+    public Object getReloadLock()
+    {
+        return reloadLock;
+    }
+
+
     /**
      * Load the configuration from the underlying location.
      *
@@ -210,7 +242,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
     {
         try
         {
-            URL url = ConfigurationUtils.locate(basePath, fileName);
+            URL url = ConfigurationUtils.locate(this.fileSystem, basePath, fileName);
 
             if (url == null)
             {
@@ -272,18 +304,11 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
             sourceURL = url;
         }
 
-        // throw an exception if the target URL is a directory
-        File file = ConfigurationUtils.fileFromURL(url);
-        if (file != null && file.isDirectory())
-        {
-            throw new ConfigurationException("Cannot load a configuration from a directory");
-        }
-
         InputStream in = null;
 
         try
         {
-            in = url.openStream();
+            in = fileSystem.getInputStream(url);
             load(in);
         }
         catch (ConfigurationException e)
@@ -395,12 +420,19 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
     {
         try
         {
-            File file = ConfigurationUtils.getFile(basePath, fileName);
+            URL url = this.fileSystem.getURL(basePath, fileName);
+
+            if (url == null)
+            {
+                throw new ConfigurationException("Cannot locate configuration source " + fileName);
+            }
+            save(url);
+            /*File file = ConfigurationUtils.getFile(basePath, fileName);
             if (file == null)
             {
                 throw new ConfigurationException("Invalid file name for save: " + fileName);
             }
-            save(file);
+            save(file); */
         }
         catch (ConfigurationException e)
         {
@@ -423,50 +455,23 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
      */
     public void save(URL url) throws ConfigurationException
     {
-        // file URLs have to be converted to Files since FileURLConnection is
-        // read only (http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4191800)
-        File file = ConfigurationUtils.fileFromURL(url);
-        if (file != null)
+        OutputStream out = null;
+        try
         {
-            save(file);
+            out = fileSystem.getOutputStream(url);
+            save(out);
+            if (out instanceof VerifiableOutputStream)
+            {
+                ((VerifiableOutputStream) out).verify();
+            }
         }
-        else
+        catch (IOException e)
         {
-            // for non file URLs save through an URLConnection
-            OutputStream out = null;
-            try
-            {
-                URLConnection connection = url.openConnection();
-                connection.setDoOutput(true);
-
-                // use the PUT method for http URLs
-                if (connection instanceof HttpURLConnection)
-                {
-                    HttpURLConnection conn = (HttpURLConnection) connection;
-                    conn.setRequestMethod("PUT");
-                }
-
-                out = connection.getOutputStream();
-                save(out);
-
-                // check the response code for http URLs and throw an exception if an error occured
-                if (connection instanceof HttpURLConnection)
-                {
-                    HttpURLConnection conn = (HttpURLConnection) connection;
-                    if (conn.getResponseCode() >= HttpURLConnection.HTTP_BAD_REQUEST)
-                    {
-                        throw new IOException("HTTP Error " + conn.getResponseCode() + " " + conn.getResponseMessage());
-                    }
-                }
-            }
-            catch (IOException e)
-            {
-                throw new ConfigurationException("Could not save to URL " + url, e);
-            }
-            finally
-            {
-                closeSilent(out);
-            }
+            throw new ConfigurationException("Could not save to URL " + url, e);
+        }
+        finally
+        {
+            closeSilent(out);
         }
     }
 
@@ -485,15 +490,9 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
 
         try
         {
-            // create the file if necessary
-            createPath(file);
-            out = new FileOutputStream(file);
+            out = fileSystem.getOutputStream(file);
             save(out);
         }
-        catch (IOException e)
-        {
-            throw new ConfigurationException("Unable to save the configuration to the file " + file, e);
-        }
         finally
         {
             closeSilent(out);
@@ -567,8 +566,14 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
      */
     public void setFileName(String fileName)
     {
+        if (fileName != null && fileName.startsWith(FILE_SCHEME) && !fileName.startsWith("file://"))
+        {
+            fileName = "file://" + fileName.substring(FILE_SCHEME.length());
+        }
+
         sourceURL = null;
         this.fileName = fileName;
+        getLogger().debug("FileName set to " + fileName);
     }
 
     /**
@@ -600,8 +605,13 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
      */
     public void setBasePath(String basePath)
     {
+        if (basePath != null && basePath.startsWith(FILE_SCHEME) && !basePath.startsWith("file://"))
+        {
+            basePath = "file://" + basePath.substring(FILE_SCHEME.length());
+        }
         sourceURL = null;
         this.basePath = basePath;
+        getLogger().debug("Base path set to " + basePath);
     }
 
     /**
@@ -654,36 +664,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
      */
     public String getPath()
     {
-        String path = null;
-        File file = getFile();
-        // if resource was loaded from jar file may be null
-        if (file != null)
-        {
-            path = file.getAbsolutePath();
-        }
-
-        // try to see if file was loaded from a jar
-        if (path == null)
-        {
-            if (sourceURL != null)
-            {
-                path = sourceURL.getPath();
-            }
-            else
-            {
-                try
-                {
-                    path = ConfigurationUtils.getURL(getBasePath(), getFileName()).getPath();
-                }
-                catch (MalformedURLException e)
-                {
-                    // simply ignore it and return null
-                    ;
-                }
-            }
-        }
-
-        return path;
+        return fileSystem.getPath(getFile(), sourceURL, getBasePath(), getFileName());
     }
 
     /**
@@ -701,6 +682,11 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
         setFile(new File(path));
     }
 
+    URL getSourceURL()
+    {
+        return sourceURL;
+    }
+
     /**
      * Return the URL where the configuration is stored.
      *
@@ -709,7 +695,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
     public URL getURL()
     {
         return (sourceURL != null) ? sourceURL
-                : ConfigurationUtils.locate(getBasePath(), getFileName());
+                : ConfigurationUtils.locate(this.fileSystem, getBasePath(), getFileName());
     }
 
     /**
@@ -725,6 +711,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
         setBasePath(ConfigurationUtils.getBasePath(url));
         setFileName(ConfigurationUtils.getFileName(url));
         sourceURL = url;
+        getLogger().debug("URL set to " + url);
     }
 
     public void setAutoSave(boolean autoSave)
@@ -765,8 +752,11 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
      */
     public void addProperty(String key, Object value)
     {
-        super.addProperty(key, value);
-        possiblySave();
+        synchronized (reloadLock)
+        {
+            super.addProperty(key, value);
+            possiblySave();
+        }
     }
 
     /**
@@ -779,14 +769,20 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
      */
     public void setProperty(String key, Object value)
     {
-        super.setProperty(key, value);
-        possiblySave();
+        synchronized (reloadLock)
+        {
+            super.setProperty(key, value);
+            possiblySave();
+        }
     }
 
     public void clearProperty(String key)
     {
-        super.clearProperty(key);
-        possiblySave();
+        synchronized (reloadLock)
+        {
+            super.clearProperty(key);
+            possiblySave();
+        }
     }
 
     public ReloadingStrategy getReloadingStrategy()
@@ -812,6 +808,11 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
      * event.
      */
     public void reload()
+    {
+        reload(false);
+    }
+
+    public boolean reload(boolean checkReload)
     {
         synchronized (reloadLock)
         {
@@ -827,21 +828,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
                         {
                             getLogger().info("Reloading configuration. URL is " + getURL());
                         }
-                        fireEvent(EVENT_RELOAD, null, getURL(), true);
-                        setDetailEvents(false);
-                        boolean autoSaveBak = this.isAutoSave(); // save the current state
-                        this.setAutoSave(false); // deactivate autoSave to prevent information loss
-                        try
-                        {
-                            clear();
-                            load();
-                        }
-                        finally
-                        {
-                            this.setAutoSave(autoSaveBak); // set autoSave to previous value
-                            setDetailEvents(true);
-                        }
-                        fireEvent(EVENT_RELOAD, null, getURL(), false);
+                        refresh();
 
                         // notify the strategy
                         strategy.reloadingPerformed();
@@ -851,6 +838,10 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
                 {
                     fireError(EVENT_RELOAD, null, null, e);
                     // todo rollback the changes if the file can't be reloaded
+                    if (checkReload)
+                    {
+                        return false;
+                    }
                 }
                 finally
                 {
@@ -858,6 +849,44 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
                 }
             }
         }
+        return true;
+    }
+
+    /**
+     * Reloads the associated configuration file. This method first clears the
+     * content of this configuration, then the associated configuration file is
+     * loaded again. Updates on this configuration which have not yet been saved
+     * are lost. Calling this method is like invoking <code>reload()</code>
+     * without checking the reloading strategy.
+     *
+     * @throws ConfigurationException if an error occurs
+     * @since 1.7
+     */
+    public void refresh() throws ConfigurationException
+    {
+        fireEvent(EVENT_RELOAD, null, getURL(), true);
+        setDetailEvents(false);
+        boolean autoSaveBak = this.isAutoSave(); // save the current state
+        this.setAutoSave(false); // deactivate autoSave to prevent information loss
+        try
+        {
+            clear();
+            load();
+        }
+        finally
+        {
+            this.setAutoSave(autoSaveBak); // set autoSave to previous value
+            setDetailEvents(true);
+        }
+        fireEvent(EVENT_RELOAD, null, getURL(), false);
+    }
+
+    /**
+     * Send notification that the configuration has changed.
+     */
+    public void configurationChanged()
+    {
+        fireEvent(EVENT_CONFIG_CHANGED, null, getURL(), true);
     }
 
     /**
@@ -930,13 +959,19 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
     public boolean isEmpty()
     {
         reload();
-        return super.isEmpty();
+        synchronized (reloadLock)
+        {
+            return super.isEmpty();
+        }
     }
 
     public boolean containsKey(String key)
     {
         reload();
-        return super.containsKey(key);
+        synchronized (reloadLock)
+        {
+            return super.containsKey(key);
+        }
     }
 
     /**
@@ -974,27 +1009,6 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
         }
     }
 
-    /**
-     * Create the path to the specified file.
-     *
-     * @param file the target file
-     */
-    private void createPath(File file)
-    {
-        if (file != null)
-        {
-            // create the path to the file if the file doesn't exist
-            if (!file.exists())
-            {
-                File parent = file.getParentFile();
-                if (parent != null && !parent.exists())
-                {
-                    parent.mkdirs();
-                }
-            }
-        }
-    }
-
     public String getEncoding()
     {
         return encoding;
@@ -1040,7 +1054,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
      * @param out the output stream to be closed (may be <b>null</b>)
      * @since 1.5
      */
-    private void closeSilent(OutputStream out)
+    protected void closeSilent(OutputStream out)
     {
         try
         {
