@@ -16,6 +16,8 @@
  */
 package org.apache.commons.configuration;
 
+import java.io.ByteArrayOutputStream;
+import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
@@ -32,6 +34,7 @@ import org.apache.commons.configuration.tree.DefaultConfigurationNode;
 import org.apache.commons.configuration.tree.DefaultExpressionEngine;
 import org.apache.commons.configuration.tree.ExpressionEngine;
 import org.apache.commons.configuration.tree.NodeCombiner;
+import org.apache.commons.configuration.tree.TreeUtils;
 import org.apache.commons.configuration.tree.UnionCombiner;
 import org.apache.commons.configuration.tree.ViewNode;
 
@@ -164,9 +167,9 @@ import org.apache.commons.configuration.tree.ViewNode;
  * href="http://commons.apache.org/configuration/team-list.html">Commons
  * Configuration team</a>
  * @since 1.3
- * @version $Id: CombinedConfiguration.java 712401 2008-11-08 15:29:56Z oheger $
+ * @version $Id: CombinedConfiguration.java 1158115 2011-08-16 06:04:32Z oheger $
  */
-public class CombinedConfiguration extends HierarchicalConfiguration implements
+public class CombinedConfiguration extends HierarchicalReloadableConfiguration implements
         ConfigurationListener, Cloneable
 {
     /**
@@ -201,6 +204,12 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
     /** Stores a map with the named configurations. */
     private Map namedConfigurations;
 
+    /** The default behavior is to ignore exceptions that occur during reload */
+    private boolean ignoreReloadExceptions = true;
+
+    /** Set to true when the backing file has changed */
+    private boolean reloadRequired;
+
     /**
      * An expression engine used for converting child configurations to
      * hierarchical ones.
@@ -223,6 +232,18 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
         clear();
     }
 
+    public CombinedConfiguration(NodeCombiner comb, Lock lock)
+    {
+        super(lock);
+        setNodeCombiner((comb != null) ? comb : DEFAULT_COMBINER);
+        clear();
+    }
+
+    public CombinedConfiguration(Lock lock)
+    {
+        this(null, lock);
+    }
+
     /**
      * Creates a new instance of <code>CombinedConfiguration</code> that uses
      * a union combiner.
@@ -231,7 +252,7 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
      */
     public CombinedConfiguration()
     {
-        this(null);
+        this(null, null);
     }
 
     /**
@@ -326,6 +347,26 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
         this.conversionExpressionEngine = conversionExpressionEngine;
     }
 
+    /**
+     * Retrieves the value of the ignoreReloadExceptions flag.
+     * @return true if exceptions are ignored, false otherwise.
+     */
+    public boolean isIgnoreReloadExceptions()
+    {
+        return ignoreReloadExceptions;
+    }
+
+    /**
+     * If set to true then exceptions that occur during reloading will be
+     * ignored. If false then the exceptions will be allowed to be thrown
+     * back to the caller.
+     * @param ignoreReloadExceptions true if exceptions should be ignored.
+     */
+    public void setIgnoreReloadExceptions(boolean ignoreReloadExceptions)
+    {
+        this.ignoreReloadExceptions = ignoreReloadExceptions;
+    }
+
     /**
      * Adds a new configuration to this combined configuration. It is possible
      * (but not mandatory) to give the new configuration a name. This name must
@@ -359,6 +400,10 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
         }
 
         ConfigData cd = new ConfigData(config, name, at);
+        if (getLogger().isDebugEnabled())
+        {
+            getLogger().debug("Adding configuration " + config + " with name " + name);
+        }
         configurations.add(cd);
         if (name != null)
         {
@@ -431,6 +476,40 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
         return (Configuration) namedConfigurations.get(name);
     }
 
+    /**
+     * Returns a List of all the configurations that have been added.
+     * @return A List of all the configurations.
+     * @since 1.7
+     */
+    public List getConfigurations()
+    {
+        List list = new ArrayList();
+        Iterator iter = configurations.iterator();
+        while (iter.hasNext())
+        {
+            list.add(((ConfigData) iter.next()).getConfiguration());
+        }
+        return list;
+    }
+
+    /**
+     * Returns a List of the names of all the configurations that have been
+     * added in the order they were added. A NULL value will be present in
+     * the list for each configuration that was added without a name.
+     * @return A List of all the configuration names.
+     * @since 1.7
+     */
+    public List getConfigurationNameList()
+    {
+        List list = new ArrayList();
+        Iterator iter = configurations.iterator();
+        while (iter.hasNext())
+        {
+            list.add(((ConfigData) iter.next()).getName());
+        }
+        return list;
+    }
+
     /**
      * Removes the specified configuration from this combined configuration.
      *
@@ -509,7 +588,7 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
      */
     public void invalidate()
     {
-        combinedRoot = null;
+        reloadRequired = true;
         fireEvent(EVENT_COMBINED_INVALIDATE, null, null, false);
     }
 
@@ -522,7 +601,11 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
      */
     public void configurationChanged(ConfigurationEvent event)
     {
-        if (!event.isBeforeUpdate())
+        if (event.getType() == AbstractFileConfiguration.EVENT_CONFIG_CHANGED)
+        {
+            fireEvent(event.getType(), event.getPropertyName(), event.getPropertyValue(), event.isBeforeUpdate());
+        }
+        else if (!event.isBeforeUpdate())
         {
             invalidate();
         }
@@ -537,11 +620,15 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
      */
     public ConfigurationNode getRootNode()
     {
-        if (combinedRoot == null)
+        synchronized (getReloadLock())
         {
-            combinedRoot = constructCombinedNode();
+            if (reloadRequired || combinedRoot == null)
+            {
+                combinedRoot = constructCombinedNode();
+                reloadRequired = false;
+            }
+            return combinedRoot;
         }
-        return combinedRoot;
     }
 
     /**
@@ -675,8 +762,10 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
             }
             catch (Exception ex)
             {
-                // ignore all exceptions, e.g. missing property exceptions
-                ;
+                if (!ignoreReloadExceptions)
+                {
+                    throw new ConfigurationRuntimeException(ex);
+                }
             }
         }
     }
@@ -690,6 +779,10 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
     {
         if (getNumberOfConfigurations() < 1)
         {
+            if (getLogger().isDebugEnabled())
+            {
+                getLogger().debug("No configurations defined for " + this);
+            }
             return new ViewNode();
         }
 
@@ -703,6 +796,13 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
                 node = getNodeCombiner().combine(node,
                         ((ConfigData) it.next()).getTransformedRoot());
             }
+            if (getLogger().isDebugEnabled())
+            {
+                ByteArrayOutputStream os = new ByteArrayOutputStream();
+                PrintStream stream = new PrintStream(os);
+                TreeUtils.printTree(stream, node);
+                getLogger().debug(os.toString());
+            }
             return node;
         }
     }
@@ -715,23 +815,26 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
      */
     private Configuration findSourceConfiguration(ConfigurationNode node)
     {
-        ConfigurationNode root = null;
-        ConfigurationNode current = node;
-
-        // find the root node in this hierarchy
-        while (current != null)
+        synchronized (getReloadLock())
         {
-            root = current;
-            current = current.getParentNode();
-        }
+            ConfigurationNode root = null;
+            ConfigurationNode current = node;
 
-        // Check with the root nodes of the child configurations
-        for (Iterator it = configurations.iterator(); it.hasNext();)
-        {
-            ConfigData cd = (ConfigData) it.next();
-            if (root == cd.getRootNode())
+            // find the root node in this hierarchy
+            while (current != null)
             {
-                return cd.getConfiguration();
+                root = current;
+                current = current.getParentNode();
+            }
+
+            // Check with the root nodes of the child configurations
+            for (Iterator it = configurations.iterator(); it.hasNext();)
+            {
+                ConfigData cd = (ConfigData) it.next();
+                if (root == cd.getRootNode())
+                {
+                    return cd.getConfiguration();
+                }
             }
         }
 
@@ -841,12 +944,12 @@ public class CombinedConfiguration extends HierarchicalConfiguration implements
             }
 
             // Copy data of the root node to the new path
-            HierarchicalConfiguration hc = ConfigurationUtils
+            ConfigurationNode root = ConfigurationUtils
                     .convertToHierarchical(getConfiguration(),
-                            getConversionExpressionEngine());
-            atParent.appendChildren(hc.getRootNode());
-            atParent.appendAttributes(hc.getRootNode());
-            rootNode = hc.getRootNode();
+                            getConversionExpressionEngine()).getRootNode();
+            atParent.appendChildren(root);
+            atParent.appendAttributes(root);
+            rootNode = root;
 
             return result;
         }
