@@ -30,11 +30,20 @@ import org.apache.commons.configuration.beanutils.BeanFactory;
 import org.apache.commons.configuration.beanutils.BeanHelper;
 import org.apache.commons.configuration.beanutils.DefaultBeanFactory;
 import org.apache.commons.configuration.beanutils.XMLBeanDeclaration;
+import org.apache.commons.configuration.event.ConfigurationErrorListener;
+import org.apache.commons.configuration.event.ConfigurationListener;
+import org.apache.commons.configuration.interpol.ConfigurationInterpolator;
+import org.apache.commons.configuration.resolver.CatalogResolver;
+import org.apache.commons.configuration.resolver.EntityRegistry;
+import org.apache.commons.configuration.resolver.EntityResolverSupport;
 import org.apache.commons.configuration.tree.ConfigurationNode;
 import org.apache.commons.configuration.tree.DefaultExpressionEngine;
 import org.apache.commons.configuration.tree.OverrideCombiner;
 import org.apache.commons.configuration.tree.UnionCombiner;
+import org.apache.commons.lang.text.StrLookup;
+import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+import org.xml.sax.EntityResolver;
 
 /**
  * <p>
@@ -51,7 +60,7 @@ import org.apache.commons.logging.LogFactory;
  * <code>beanutils</code> package (namely
  * <code>{@link org.apache.commons.configuration.beanutils.XMLBeanDeclaration XMLBeanDeclaration}</code>
  * will be used to extract the configuration's initialization parameters, which
- * allows for complex initialization szenarios).
+ * allows for complex initialization scenarios).
  * </p>
  * <p>
  * It is also possible to add custom tags to the configuration definition file.
@@ -67,7 +76,7 @@ import org.apache.commons.logging.LogFactory;
  * <p>
  *
  * <pre>
- * &lt;configuration&gt;
+ * &lt;configuration systemProperties="properties file name"&gt;
  *   &lt;header&gt;
  *     &lt;!-- Optional meta information about the composite configuration --&gt;
  *   &lt;/header&gt;
@@ -83,7 +92,13 @@ import org.apache.commons.logging.LogFactory;
  * </p>
  * <p>
  * The name of the root element (here <code>configuration</code>) is
- * arbitrary. There are two sections (both of them are optional) for declaring
+ * arbitrary. The optional systemProperties attribute identifies the path to
+ * a property file containing properties that should be added to the system
+ * properties. If specified on the root element, the system properties are
+ * set before the rest of the configuration is processed.
+ * </p>
+ * <p>
+ * There are two sections (both of them are optional) for declaring
  * <em>override</em> and <em>additional</em> configurations. Configurations
  * in the former section are evaluated in the order of their declaration, and
  * properties of configurations declared earlier hide those of configurations
@@ -137,7 +152,7 @@ import org.apache.commons.logging.LogFactory;
  * the resulting configuration.
  * </p>
  * <p>
- * The configuration object returned by this builder is an instance of the
+ * The default configuration object returned by this builder is an instance of the
  * <code>{@link CombinedConfiguration}</code> class. The return value of the
  * <code>getConfiguration()</code> method can be casted to this type, and the
  * <code>getConfiguration(boolean)</code> method directly declares
@@ -146,7 +161,37 @@ import org.apache.commons.logging.LogFactory;
  * configuration (e.g. for updates of single configuration objects). It has also
  * the advantage that the properties stored in all declared configuration
  * objects are collected and transformed into a single hierarchical structure,
- * which can be accessed using different expression engines.
+ * which can be accessed using different expression engines. The actual CombinedConfiguration
+ * implementation can be overridden by specifying the class in the <em>config-class</em>
+ * attribute of the result element.
+ * </p>
+ * <p>
+ * A custom EntityResolver can be used for all XMLConfigurations by adding
+ * <pre>
+ * &lt;entity-resolver config-class="EntityResolver fully qualified class name"&gt;
+ * </pre>
+ * The CatalogResolver can be used for all XMLConfiguration by adding
+ * <pre>
+ * &lt;entity-resolver catalogFiles="comma separated list of catalog files"&gt;
+ * </pre>
+ * </p>
+ * <p>
+ * Additional ConfigurationProviders can be added by configuring them in the <em>header</em>
+ * section.
+ * <pre>
+ * &lt;providers&gt;
+ *   &lt;provider config-tag="tag name" config-class="provider fully qualified class name"/&gt;
+ * &lt;/providers&gt;
+ * </pre>
+ * </p>
+ * <p>
+ * Additional variable resolvers can be added by configuring them in the <em>header</em>
+ * section.
+ * <pre>
+ * &lt;lookups&gt;
+ *   &lt;lookup config-prefix="prefix" config-class="StrLookup fully qualified class name"/&gt;
+ * &lt;/lookups&gt;
+ * </pre>
  * </p>
  * <p>
  * All declared override configurations are directly added to the resulting
@@ -154,7 +199,7 @@ import org.apache.commons.logging.LogFactory;
  * <code>config-name</code> attribute), they can directly be accessed using
  * the <code>getConfiguration(String)</code> method of
  * <code>CombinedConfiguration</code>. The additional configurations are
- * alltogether added to another combined configuration, which uses a union
+ * altogether added to another combined configuration, which uses a union
  * combiner. Then this union configuration is added to the resulting combined
  * configuration under the name defined by the <code>ADDITIONAL_NAME</code>
  * constant.
@@ -169,7 +214,7 @@ import org.apache.commons.logging.LogFactory;
  * @author <a
  * href="http://commons.apache.org/configuration/team-list.html">Commons
  * Configuration team</a>
- * @version $Id: DefaultConfigurationBuilder.java 595043 2007-11-14 20:44:43Z oheger $
+ * @version $Id: DefaultConfigurationBuilder.java 1158119 2011-08-16 06:20:57Z oheger $
  */
 public class DefaultConfigurationBuilder extends XMLConfiguration implements
         ConfigurationBuilder
@@ -237,6 +282,11 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
             + "forceCreate"
             + DefaultExpressionEngine.DEFAULT_ATTRIBUTE_END;
 
+    /**
+     * Constant for the tag attribute for providers.
+     */
+    static final String KEY_SYSTEM_PROPS = "[@systemProperties]";
+
     /** Constant for the name of the header section. */
     static final String SEC_HEADER = "header";
 
@@ -268,6 +318,38 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
     static final String KEY_ADDITIONAL_LIST = SEC_HEADER
             + ".combiner.additional.list-nodes.node";
 
+    /**
+     * Constant for the key for defining providers in the configuration file.
+     */
+    static final String KEY_CONFIGURATION_PROVIDERS = SEC_HEADER
+            + ".providers.provider";
+
+    /**
+     * Constant for the tag attribute for providers.
+     */
+    static final String KEY_PROVIDER_KEY = XMLBeanDeclaration.ATTR_PREFIX + "tag]";
+
+    /**
+     * Constant for the key for defining variable resolvers
+     */
+    static final String KEY_CONFIGURATION_LOOKUPS = SEC_HEADER
+            + ".lookups.lookup";
+
+    /**
+     * Constant for the key for defining entity resolvers
+     */
+    static final String KEY_ENTITY_RESOLVER = SEC_HEADER + ".entity-resolver";
+
+    /**
+     * Constant for the prefix attribute for lookups.
+     */
+    static final String KEY_LOOKUP_KEY = XMLBeanDeclaration.ATTR_PREFIX + "prefix]";
+
+    /**
+     * Constance for the FileSystem.
+     */
+    static final String FILE_SYSTEM = SEC_HEADER + ".fileSystem";
+
     /**
      * Constant for the key of the result declaration. This key can point to a
      * bean declaration, which defines properties of the resulting combined
@@ -297,6 +379,14 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
     private static final ConfigurationProvider SYSTEM_PROVIDER = new ConfigurationProvider(
             SystemConfiguration.class);
 
+    /** Constant for the provider for ini files. */
+    private static final ConfigurationProvider INI_PROVIDER =
+            new FileConfigurationProvider(HierarchicalINIConfiguration.class);
+
+    /** Constant for the provider for environment properties. */
+    private static final ConfigurationProvider ENV_PROVIDER =
+            new ConfigurationProvider(EnvironmentConfiguration.class);
+
     /** Constant for the provider for plist files. */
     private static final ConfigurationProvider PLIST_PROVIDER = new FileExtensionConfigurationProvider(
             "org.apache.commons.configuration.plist.XMLPropertyListConfiguration",
@@ -307,13 +397,17 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
     private static final ConfigurationProvider BUILDER_PROVIDER = new ConfigurationBuilderProvider();
 
     /** An array with the names of the default tags. */
-    private static final String[] DEFAULT_TAGS =
-    {"properties", "xml", "hierarchicalXml", "jndi", "system", "plist", "configuration"};
+    private static final String[] DEFAULT_TAGS = {
+            "properties", "xml", "hierarchicalXml", "jndi", "system", "plist",
+            "configuration", "ini", "env"
+    };
 
     /** An array with the providers for the default tags. */
-    private static final ConfigurationProvider[] DEFAULT_PROVIDERS =
-    {PROPERTIES_PROVIDER, XML_PROVIDER, XML_PROVIDER, JNDI_PROVIDER,
-            SYSTEM_PROVIDER, PLIST_PROVIDER, BUILDER_PROVIDER};
+    private static final ConfigurationProvider[] DEFAULT_PROVIDERS = {
+            PROPERTIES_PROVIDER, XML_PROVIDER, XML_PROVIDER, JNDI_PROVIDER,
+            SYSTEM_PROVIDER, PLIST_PROVIDER, BUILDER_PROVIDER, INI_PROVIDER,
+            ENV_PROVIDER
+    };
 
     /**
      * The serial version UID.
@@ -399,9 +493,9 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
     /**
      * Sets the base path for the configuration sources to load. Normally a base
      * path need not to be set because it is determined by the location of the
-     * configuration definition file to load. All relative pathes in this file
+     * configuration definition file to load. All relative paths in this file
      * are resolved relative to this file. Setting a base path makes sense if
-     * such relative pathes should be otherwise resolved, e.g. if the
+     * such relative paths should be otherwise resolved, e.g. if the
      * configuration file is loaded from the class path and all sub
      * configurations it refers to are stored in a special config directory.
      *
@@ -493,6 +587,12 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
             load();
         }
 
+        initFileSystem();
+        initSystemProperties();
+        configureEntityResolver();
+        registerConfiguredProviders();
+        registerConfiguredLookups();
+
         CombinedConfiguration result = createResultConfiguration();
         constructedConfiguration = result;
 
@@ -503,8 +603,7 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
         List additionals = fetchChildConfigs(KEY_UNION);
         if (!additionals.isEmpty())
         {
-            CombinedConfiguration addConfig = new CombinedConfiguration(
-                    new UnionCombiner());
+            CombinedConfiguration addConfig = createAdditionalsConfiguration(result);
             result.addConfiguration(addConfig, ADDITIONAL_NAME);
             initCombinedConfiguration(addConfig, additionals,
                     KEY_ADDITIONAL_LIST);
@@ -540,13 +639,39 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
         return result;
     }
 
+    /**
+     * Creates the <code>CombinedConfiguration</code> for the configuration
+     * sources in the <code>&lt;additional&gt;</code> section. This method is
+     * called when the builder constructs the final configuration. It creates a
+     * new <code>CombinedConfiguration</code> and initializes some properties
+     * from the result configuration.
+     *
+     * @param resultConfig the result configuration (this is the configuration
+     *        that will be returned by the builder)
+     * @return the <code>CombinedConfiguration</code> for the additional
+     *         configuration sources
+     * @since 1.7
+     */
+    protected CombinedConfiguration createAdditionalsConfiguration(
+            CombinedConfiguration resultConfig)
+    {
+        CombinedConfiguration addConfig =
+                new CombinedConfiguration(new UnionCombiner());
+        addConfig.setDelimiterParsingDisabled(resultConfig
+                .isDelimiterParsingDisabled());
+        addConfig.setForceReloadCheck(resultConfig.isForceReloadCheck());
+        addConfig.setIgnoreReloadExceptions(resultConfig
+                .isIgnoreReloadExceptions());
+        return addConfig;
+    }
+
     /**
      * Initializes a combined configuration for the configurations of a specific
      * section. This method is called for the override and for the additional
      * section (if it exists).
      *
      * @param config the configuration to be initialized
-     * @param containedConfigs the list with the declaratinos of the contained
+     * @param containedConfigs the list with the declarations of the contained
      * configurations
      * @param keyListNodes a list with the declaration of list nodes
      * @throws ConfigurationException if an error occurs
@@ -566,6 +691,11 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
                     .next();
             ConfigurationDeclaration decl = new ConfigurationDeclaration(this,
                     conf);
+            if (getLogger().isDebugEnabled())
+            {
+                getLogger().debug("Creating configuration " + decl.getBeanClassName() + " with name "
+                    + decl.getConfiguration().getString(ATTR_NAME));
+            }
             AbstractConfiguration newConf = createConfigurationAt(decl);
             if (newConf != null)
             {
@@ -588,6 +718,88 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
         }
     }
 
+    /**
+     * Registers providers defined in the configuration.
+     *
+     * @throws ConfigurationException if an error occurs
+     */
+    protected void registerConfiguredProviders() throws ConfigurationException
+    {
+        List nodes = configurationsAt(KEY_CONFIGURATION_PROVIDERS);
+        for (Iterator it = nodes.iterator(); it.hasNext();)
+        {
+            HierarchicalConfiguration config = (HierarchicalConfiguration) it.next();
+            XMLBeanDeclaration decl = new XMLBeanDeclaration(config);
+            String key = config.getString(KEY_PROVIDER_KEY);
+            addConfigurationProvider(key, (ConfigurationProvider) BeanHelper
+                    .createBean(decl));
+        }
+    }
+
+    /**
+     * Registers StrLookups defined in the configuration.
+     *
+     * @throws ConfigurationException if an error occurs
+     */
+    protected void registerConfiguredLookups() throws ConfigurationException
+    {
+        List nodes = configurationsAt(KEY_CONFIGURATION_LOOKUPS);
+        for (Iterator it = nodes.iterator(); it.hasNext();)
+        {
+            HierarchicalConfiguration config = (HierarchicalConfiguration) it.next();
+            XMLBeanDeclaration decl = new XMLBeanDeclaration(config);
+            String key = config.getString(KEY_LOOKUP_KEY);
+            StrLookup lookup = (StrLookup) BeanHelper.createBean(decl);
+            BeanHelper.setProperty(lookup, "configuration", this);
+            ConfigurationInterpolator.registerGlobalLookup(key, lookup);
+            this.getInterpolator().registerLookup(key, lookup);
+        }
+    }
+
+    protected void initFileSystem() throws ConfigurationException
+    {
+        if (getMaxIndex(FILE_SYSTEM) == 0)
+        {
+            HierarchicalConfiguration config = configurationAt(FILE_SYSTEM);
+            XMLBeanDeclaration decl = new XMLBeanDeclaration(config);
+            setFileSystem((FileSystem) BeanHelper.createBean(decl));
+        }
+    }
+
+    /**
+     * If a property file is configured add the properties to the System properties.
+     * @throws ConfigurationException if an error occurs.
+     */
+    protected void initSystemProperties() throws ConfigurationException
+    {
+        String fileName = getString(KEY_SYSTEM_PROPS);
+        if (fileName != null)
+        {
+            try
+            {
+               SystemConfiguration.setSystemProperties(getConfigurationBasePath(), fileName);
+            }
+            catch (Exception ex)
+            {
+                throw new ConfigurationException("Error setting system properties from " + fileName, ex);
+            }
+
+        }
+    }
+
+    protected void configureEntityResolver() throws ConfigurationException
+    {
+        if (getMaxIndex(KEY_ENTITY_RESOLVER) == 0)
+        {
+            XMLBeanDeclaration decl = new XMLBeanDeclaration(this, KEY_ENTITY_RESOLVER, true);
+            EntityResolver resolver = (EntityResolver) BeanHelper.createBean(decl, CatalogResolver.class);
+            BeanHelper.setProperty(resolver, "fileSystem", getFileSystem());
+            BeanHelper.setProperty(resolver, "baseDir", getBasePath());
+            BeanHelper.setProperty(resolver, "substitutor", getSubstitutor());
+            setEntityResolver(resolver);
+        }
+    }
+
     /**
      * Performs interpolation. This method will not only take this configuration
      * instance into account (which is the one that loaded the configuration
@@ -712,7 +924,7 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
      */
     private void registerBeanFactory()
     {
-        synchronized (getClass())
+        synchronized (DefaultConfigurationBuilder.class)
         {
             if (!BeanHelper.registeredFactoryNames().contains(
                     CONFIG_BEAN_FACTORY_NAME))
@@ -1058,6 +1270,9 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
      */
     static class ConfigurationBeanFactory implements BeanFactory
     {
+        /** The logger. */
+        private Log logger = LogFactory.getLog(DefaultConfigurationBuilder.class);
+
         /**
          * Creates an instance of a bean class. This implementation expects that
          * the passed in bean declaration is a declaration for a configuration.
@@ -1101,6 +1316,11 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
                 }
                 else
                 {
+                    if (logger.isDebugEnabled())
+                    {
+                        logger.debug("Load failed for optional configuration " + tagName + ": "
+                            + ex.getMessage());
+                    }
                     // Notify registered error listeners
                     decl.getConfigurationBuilder().fireError(
                             EVENT_ERR_LOAD_OPTIONAL,
@@ -1188,6 +1408,14 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
                 ConfigurationDeclaration decl) throws Exception
         {
             AbstractConfiguration result = getEmptyConfiguration(decl);
+            if (result instanceof FileSystemBased)
+            {
+                DefaultConfigurationBuilder builder = decl.getConfigurationBuilder();
+                if (builder.getFileSystem() != null)
+                {
+                    ((FileSystemBased) result).setFileSystem(builder.getFileSystem());
+                }
+            }
             ((FileConfiguration) result).load();
             return result;
         }
@@ -1212,7 +1440,20 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
         public AbstractConfiguration getEmptyConfiguration(
                 ConfigurationDeclaration decl) throws Exception
         {
-            return super.getConfiguration(decl);
+            AbstractConfiguration config = super.getConfiguration(decl);
+
+            /**
+             * Some wrapper classes may need to pass the EntityResolver to XMLConfigurations
+             * they construct buy may not be an XMLConfiguration.
+             */
+            if (config instanceof EntityResolverSupport)
+            {
+                DefaultConfigurationBuilder builder = decl.getConfigurationBuilder();
+                EntityResolver resolver = builder.getEntityResolver();
+                ((EntityResolverSupport) config).setEntityResolver(resolver);
+            }
+
+            return config;
         }
 
         /**
@@ -1239,8 +1480,10 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
      * implementation acts like a <code>FileConfigurationProvider</code>, but
      * it will copy all entity IDs that have been registered for the
      * configuration builder to the new XML configuration before it is loaded.
+     *
+     * @since 1.6
      */
-    static class XMLConfigurationProvider extends FileConfigurationProvider
+    public static class XMLConfigurationProvider extends FileConfigurationProvider
     {
         /**
          * Creates a new instance of <code>XMLConfigurationProvider</code>.
@@ -1265,11 +1508,19 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
             XMLConfiguration config = (XMLConfiguration) super
                     .getEmptyConfiguration(decl);
 
-            // copy the registered entities
             DefaultConfigurationBuilder builder = decl
                     .getConfigurationBuilder();
-            config.getRegisteredEntities().putAll(
+            EntityResolver resolver = builder.getEntityResolver();
+            if (resolver instanceof EntityRegistry)
+            {
+                // copy the registered entities
+                config.getRegisteredEntities().putAll(
                     builder.getRegisteredEntities());
+            }
+            else
+            {
+                config.setEntityResolver(resolver);
+            }
             return config;
         }
     }
@@ -1454,5 +1705,56 @@ public class DefaultConfigurationBuilder extends XMLConfiguration implements
         {
             return new CombinedConfiguration();
         }
+
+        /**
+         * {@inheritDoc} This implementation ensures that the configuration
+         * builder created by this provider inherits the properties from the
+         * current configuration builder.
+         */
+        protected void initBeanInstance(Object bean, BeanDeclaration data)
+                throws Exception
+        {
+            ConfigurationDeclaration decl = (ConfigurationDeclaration) data;
+            initChildBuilder(decl.getConfigurationBuilder(),
+                    (DefaultConfigurationBuilder) bean);
+            super.initBeanInstance(bean, data);
+        }
+
+        /**
+         * Initializes the given child configuration builder from its parent
+         * builder. This method copies the values of some properties from the
+         * parent builder to the child builder so that the child inherits
+         * properties from its parent.
+         *
+         * @param parent the parent builder
+         * @param child the child builder
+         */
+        private static void initChildBuilder(
+                DefaultConfigurationBuilder parent,
+                DefaultConfigurationBuilder child)
+        {
+            child.setAttributeSplittingDisabled(parent
+                    .isAttributeSplittingDisabled());
+            child.setBasePath(parent.getBasePath());
+            child.setDelimiterParsingDisabled(parent
+                    .isDelimiterParsingDisabled());
+            child.setListDelimiter(parent.getListDelimiter());
+            child.setThrowExceptionOnMissing(parent.isThrowExceptionOnMissing());
+            child.setLogger(parent.getLogger());
+
+            child.clearConfigurationListeners();
+            for (Iterator it = parent.getConfigurationListeners().iterator(); it
+                    .hasNext();)
+            {
+                child.addConfigurationListener((ConfigurationListener) it
+                        .next());
+            }
+            child.clearErrorListeners();
+            for (Iterator it = parent.getErrorListeners().iterator(); it
+                    .hasNext();)
+            {
+                child.addErrorListener((ConfigurationErrorListener) it.next());
+            }
+        }
     }
 }
