@@ -36,6 +36,7 @@ import org.apache.commons.configuration.tree.DefaultConfigurationNode;
 import org.apache.commons.configuration.tree.DefaultExpressionEngine;
 import org.apache.commons.configuration.tree.ExpressionEngine;
 import org.apache.commons.configuration.tree.NodeAddData;
+import org.apache.commons.configuration.tree.ViewNode;
 import org.apache.commons.lang.StringUtils;
 
 /**
@@ -123,7 +124,7 @@ import org.apache.commons.lang.StringUtils;
  * synchronization has to be performed manually.</p>
  *
  * @author Oliver Heger
- * @version $Id: HierarchicalConfiguration.java 722238 2008-12-01 21:28:31Z oheger $
+ * @version $Id: HierarchicalConfiguration.java 1158122 2011-08-16 06:21:58Z oheger $
  */
 public class HierarchicalConfiguration extends AbstractConfiguration implements Serializable, Cloneable
 {
@@ -192,6 +193,17 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
         }
     }
 
+    /**
+     * Returns the object to synchronize on a reload. This class is not
+     * reloadable so this object isn't important
+     *
+     * @return the lock object
+     */
+    public Object getReloadLock()
+    {
+        return this;
+    }
+
     /**
      * Returns the root node of this hierarchical configuration. This method
      * exists for backwards compatibility only. New code should use the
@@ -758,6 +770,20 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
         fireEvent(EVENT_SET_PROPERTY, key, value, false);
     }
 
+    /**
+     * Clears this configuration. This is a more efficient implementation than
+     * the one inherited from the base class. It directly removes all data from
+     * the root node.
+     */
+    public void clear()
+    {
+        fireEvent(EVENT_CLEAR, null, null, true);
+        getRootNode().removeAttributes();
+        getRootNode().removeChildren();
+        getRootNode().setValue(null);
+        fireEvent(EVENT_CLEAR, null, null, false);
+    }
+
     /**
      * Removes all values of the property with the given name and of keys that
      * start with this name. So if there is a property with the key
@@ -816,7 +842,10 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
     /**
      * Returns an iterator with all keys defined in this configuration that
      * start with the given prefix. The returned keys will not contain any
-     * indices.
+     * indices. This implementation tries to locate a node whose key is the same
+     * as the passed in prefix. Then the subtree of this node is traversed, and
+     * the keys of all nodes encountered (including attributes) are added to the
+     * result set.
      *
      * @param prefix the prefix of the keys to start with
      * @return an iterator with the found keys
@@ -1158,6 +1187,39 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
         });
     }
 
+    /**
+     * Transforms the specified object into a Node. This method treats view
+     * nodes in a special way. This is necessary because ViewNode does not
+     * extend HierarchicalConfiguration.Node; thus the API for the node visitor
+     * is slightly different. Therefore a view node is transformed into a
+     * special compatibility Node object.
+     *
+     * @param obj the original node object
+     * @return the node to be used
+     */
+    private static Node getNodeFor(Object obj)
+    {
+        Node nd;
+        if (obj instanceof ViewNode)
+        {
+            final ViewNode viewNode = (ViewNode) obj;
+            nd = new Node(viewNode)
+            {
+                public void setReference(Object reference)
+                {
+                    super.setReference(reference);
+                    // also set the reference at the original node
+                    viewNode.setReference(reference);
+                }
+            };
+        }
+        else
+        {
+            nd = (Node) obj;
+        }
+        return nd;
+    }
+
     /**
      * A data class for storing (hierarchical) property information. A property
      * can have a value and an arbitrary number of child properties. From
@@ -1331,19 +1393,21 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
             for (Iterator it = getChildren().iterator(); it.hasNext()
                     && !visitor.terminate();)
             {
-                ((Node) it.next()).visit(visitor, key);
+                Object obj = it.next();
+                getNodeFor(obj).visit(visitor, key);
             }
             for (Iterator it = getAttributes().iterator(); it.hasNext()
                     && !visitor.terminate();)
             {
-                ((Node) it.next()).visit(visitor, key);
+                Object obj = it.next();
+                getNodeFor(obj).visit(visitor, key);
             }
 
+            visitor.visitAfterChildren(this, key);
             if (key != null)
             {
                 key.setLength(length);
             }
-            visitor.visitAfterChildren(this, key);
         }
     }
 
@@ -1621,7 +1685,8 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
                 do
                 {
                     sibling1 = nd;
-                    nd = (Node) children.next();
+                    Object obj = children.next();
+                    nd = getNodeFor(obj);
                 } while (nd.getReference() != null && children.hasNext());
 
                 if (nd.getReference() == null)
@@ -1631,7 +1696,8 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
                     newNodes.add(nd);
                     while (children.hasNext())
                     {
-                        nd = (Node) children.next();
+                        Object obj = children.next();
+                        nd = getNodeFor(obj);
                         if (nd.getReference() == null)
                         {
                             newNodes.add(nd);
