@@ -17,14 +17,12 @@
 package org.apache.commons.configuration;
 
 import java.io.File;
-import java.io.FileNotFoundException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.Reader;
 import java.io.Writer;
 import java.math.BigDecimal;
 import java.math.BigInteger;
-import java.net.MalformedURLException;
 import java.net.URL;
 import java.util.Collection;
 import java.util.HashMap;
@@ -33,12 +31,21 @@ import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 
+import org.apache.commons.beanutils.BeanUtils;
 import org.apache.commons.configuration.event.ConfigurationErrorEvent;
 import org.apache.commons.configuration.event.ConfigurationErrorListener;
 import org.apache.commons.configuration.event.ConfigurationEvent;
 import org.apache.commons.configuration.event.ConfigurationListener;
+import org.apache.commons.configuration.interpol.ConfigurationInterpolator;
+import org.apache.commons.configuration.reloading.ReloadingStrategy;
+import org.apache.commons.configuration.resolver.EntityResolverSupport;
 import org.apache.commons.configuration.tree.ConfigurationNode;
 import org.apache.commons.configuration.tree.ExpressionEngine;
+import org.apache.commons.lang.text.StrSubstitutor;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+import org.xml.sax.EntityResolver;
+import org.xml.sax.SAXParseException;
 
 /**
  * This class provides access to multiple configuration files that reside in a location that
@@ -50,14 +57,11 @@ import org.apache.commons.configuration.tree.ExpressionEngine;
  * @author <a
  * href="http://commons.apache.org/configuration/team-list.html">Commons
  * Configuration team</a>
- * @version $Id: MultiFileHierarchicalConfiguration.java 727958 2008-12-19 07:19:24Z oheger $
+ * @version $Id: MultiFileHierarchicalConfiguration.java 1158885 2011-08-17 19:57:01Z oheger $
  */
 public class MultiFileHierarchicalConfiguration extends AbstractHierarchicalFileConfiguration
-    implements ConfigurationListener, ConfigurationErrorListener
+    implements ConfigurationListener, ConfigurationErrorListener, EntityResolverSupport
 {
-    /** FILE URL prefix */
-    private static final String FILE_URL_PREFIX = "file:";
-
     /**
      * Prevent recursion while resolving unprefixed properties.
      */
@@ -78,6 +82,30 @@ public class MultiFileHierarchicalConfiguration extends AbstractHierarchicalFile
     /** True if the constructor has finished */
     private boolean init;
 
+    /** Return an empty configuration if loading fails */
+    private boolean ignoreException = true;
+
+    /** Capture the schema validation setting */
+    private boolean schemaValidation;
+
+    /** Stores a flag whether DTD or Schema validation should be performed.*/
+    private boolean validating;
+
+    /** A flag whether attribute splitting is disabled.*/
+    private boolean attributeSplittingDisabled;
+
+    /** The Logger name to use */
+    private String loggerName = MultiFileHierarchicalConfiguration.class.getName();
+
+    /** The Reloading strategy to use on created configurations */
+    private ReloadingStrategy fileStrategy;
+
+    /** The EntityResolver */
+    private EntityResolver entityResolver;
+
+    /** The internally used helper object for variable substitution. */
+    private StrSubstitutor localSubst = new StrSubstitutor(new ConfigurationInterpolator());
+
     /**
      * Default Constructor.
      */
@@ -85,6 +113,7 @@ public class MultiFileHierarchicalConfiguration extends AbstractHierarchicalFile
     {
         super();
         this.init = true;
+        setLogger(LogFactory.getLog(loggerName));
     }
 
     /**
@@ -96,6 +125,12 @@ public class MultiFileHierarchicalConfiguration extends AbstractHierarchicalFile
         super();
         this.pattern = pathPattern;
         this.init = true;
+        setLogger(LogFactory.getLog(loggerName));
+    }
+
+    public void setLoggerName(String name)
+    {
+        this.loggerName = name;
     }
 
     /**
@@ -107,18 +142,64 @@ public class MultiFileHierarchicalConfiguration extends AbstractHierarchicalFile
         this.pattern = pathPattern;
     }
 
-    /**
-     * Creates the file configuration delegate, i.e. the object that implements
-     * functionality required by the <code>FileConfiguration</code> interface.
-     * This base implementation will return an instance of the
-     * <code>FileConfigurationDelegate</code> class. Derived classes may
-     * override it to create a different delegate object.
-     *
-     * @return the file configuration delegate
-     */
-    protected FileConfigurationDelegate createDelegate()
+    public boolean isSchemaValidation()
     {
-        return new FileConfigurationDelegate();
+        return schemaValidation;
+    }
+
+    public void setSchemaValidation(boolean schemaValidation)
+    {
+        this.schemaValidation = schemaValidation;
+    }
+
+    public boolean isValidating()
+    {
+        return validating;
+    }
+
+    public void setValidating(boolean validating)
+    {
+        this.validating = validating;
+    }
+
+    public boolean isAttributeSplittingDisabled()
+    {
+        return attributeSplittingDisabled;
+    }
+
+    public void setAttributeSplittingDisabled(boolean attributeSplittingDisabled)
+    {
+        this.attributeSplittingDisabled = attributeSplittingDisabled;
+    }
+
+    public ReloadingStrategy getReloadingStrategy()
+    {
+        return fileStrategy;
+    }
+
+    public void setReloadingStrategy(ReloadingStrategy strategy)
+    {
+        this.fileStrategy = strategy;
+    }
+
+    public void setEntityResolver(EntityResolver entityResolver)
+    {
+        this.entityResolver = entityResolver;
+    }
+
+    public EntityResolver getEntityResolver()
+    {
+        return this.entityResolver;
+    }
+
+    /**
+     * Set to true if an empty Configuration should be returned when loading fails. If
+     * false an exception will be thrown.
+     * @param ignoreException The ignore value.
+     */
+    public void setIgnoreException(boolean ignoreException)
+    {
+        this.ignoreException = ignoreException;
     }
 
     public void addProperty(String key, Object value)
@@ -329,6 +410,11 @@ public class MultiFileHierarchicalConfiguration extends AbstractHierarchicalFile
         return this.getConfiguration().subset(prefix);
     }
 
+    public Object getReloadLock()
+    {
+        return this.getConfiguration().getReloadLock();
+    }
+
     public Node getRoot()
     {
         return this.getConfiguration().getRoot();
@@ -466,7 +552,7 @@ public class MultiFileHierarchicalConfiguration extends AbstractHierarchicalFile
 
     public void load() throws ConfigurationException
     {
-        this.getConfiguration().load();
+        this.getConfiguration();
     }
 
     public void load(String fileName) throws ConfigurationException
@@ -548,6 +634,14 @@ public class MultiFileHierarchicalConfiguration extends AbstractHierarchicalFile
                 listener.configurationError(event);
             }
         }
+
+        if (event.getType() == AbstractFileConfiguration.EVENT_RELOAD)
+        {
+            if (isThrowable(event.getCause()))
+            {
+                throw new ConfigurationRuntimeException(event.getCause());
+            }
+        }
     }
 
     /*
@@ -596,7 +690,7 @@ public class MultiFileHierarchicalConfiguration extends AbstractHierarchicalFile
         {
             throw new ConfigurationRuntimeException("File pattern must be defined");
         }
-        String path = getSubstitutor().replace(pattern);
+        String path = localSubst.replace(pattern);
         synchronized (configurationsMap)
         {
             if (configurationsMap.containsKey(path))
@@ -624,58 +718,90 @@ public class MultiFileHierarchicalConfiguration extends AbstractHierarchicalFile
         }
 
         XMLConfiguration configuration = new XMLConfiguration();
+
+        if (loggerName != null)
+        {
+            Log log = LogFactory.getLog(loggerName);
+            if (log != null)
+            {
+                configuration.setLogger(log);
+            }
+        }
+        configuration.setBasePath(getBasePath());
+        configuration.setFileName(path);
+        configuration.setFileSystem(getFileSystem());
+        configuration.setExpressionEngine(getExpressionEngine());
+        ReloadingStrategy strategy = createReloadingStrategy();
+        if (strategy != null)
+        {
+            configuration.setReloadingStrategy(strategy);
+        }
+        configuration.setDelimiterParsingDisabled(isDelimiterParsingDisabled());
+        configuration.setValidating(validating);
+        configuration.setSchemaValidation(schemaValidation);
+        configuration.setEntityResolver(entityResolver);
+        configuration.setAttributeSplittingDisabled(attributeSplittingDisabled);
+        configuration.setListDelimiter(getListDelimiter());
+        configuration.addConfigurationListener(this);
+        configuration.addErrorListener(this);
+
         try
         {
-            URL url = getURL(path);
-            configuration.setURL(url);
             configuration.load();
-            configuration.setExpressionEngine(getExpressionEngine());
-            configuration.setReloadingStrategy(getReloadingStrategy());
-            configuration.addConfigurationListener(this);
-            configuration.addErrorListener(this);
-            synchronized (configurationsMap)
-            {
-                if (!configurationsMap.containsKey(path))
-                {
-                    configurationsMap.put(path, configuration);
-                }
-            }
         }
         catch (ConfigurationException ce)
         {
-            throw new ConfigurationRuntimeException(ce);
+            if (isThrowable(ce))
+            {
+                throw new ConfigurationRuntimeException(ce);
+            }
         }
-        catch (FileNotFoundException fnfe)
+        synchronized (configurationsMap)
         {
-            throw new ConfigurationRuntimeException(fnfe);
+            if (!configurationsMap.containsKey(path))
+            {
+                configurationsMap.put(path, configuration);
+            }
         }
 
         return configuration;
     }
 
-    private URL getURL(String resourceLocation) throws FileNotFoundException
+    private boolean isThrowable(Throwable throwable)
     {
-        if (resourceLocation == null)
+        if (!ignoreException)
         {
-            throw new IllegalArgumentException("A path pattern must be configured");
+            return true;
+        }
+        Throwable cause = throwable.getCause();
+        while (cause != null && !(cause instanceof SAXParseException))
+        {
+            cause = cause.getCause();
+        }
+        return cause != null;
+    }
+
+    /**
+     * Clone the FileReloadingStrategy since each file needs its own.
+     * @return A new FileReloadingStrategy.
+     */
+    private ReloadingStrategy createReloadingStrategy()
+    {
+        if (fileStrategy == null)
+        {
+            return null;
         }
         try
         {
-            // try URL
-            return new URL(resourceLocation);
+            ReloadingStrategy strategy = (ReloadingStrategy) BeanUtils.cloneBean(fileStrategy);
+            strategy.setConfiguration(null);
+            return strategy;
         }
-        catch (MalformedURLException ex)
+        catch (Exception ex)
         {
-            // no URL -> treat as file path
-            try
-            {
-                return new URL(FILE_URL_PREFIX + resourceLocation);
-            }
-            catch (MalformedURLException ex2)
-            {
-                throw new FileNotFoundException("Resource location [" + resourceLocation
-                        + "] is not a URL or a well-formed file path");
-            }
+            return null;
         }
+
     }
+
 }
