@@ -170,13 +170,22 @@ import org.apache.commons.lang.StringUtils;
  * @author <a href="mailto:epugh@upstate.com">Eric Pugh</a>
  * @author Oliver Heger
  * @author <a href="mailto:ebourg@apache.org">Emmanuel Bourg</a>
- * @version $Id: PropertiesConfiguration.java 727168 2008-12-16 21:44:29Z oheger $
+ * @version $Id: PropertiesConfiguration.java 1162387 2011-08-27 16:05:20Z oheger $
  */
 public class PropertiesConfiguration extends AbstractFileConfiguration
 {
     /** Constant for the supported comment characters.*/
     static final String COMMENT_CHARS = "#!";
 
+    /** Constant for the default properties separator.*/
+    static final String DEFAULT_SEPARATOR = " = ";
+
+    /**
+     * Constant for the default <code>IOFactory</code>. This instance is used
+     * when no specific factory was set.
+     */
+    private static final IOFactory DEFAULT_IO_FACTORY = new DefaultIOFactory();
+
     /**
      * This is the name of the property that can point to other
      * properties file for including other properties files.
@@ -201,6 +210,9 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
     /** Constant for the escaping character.*/
     private static final String ESCAPE = "\\";
 
+    /** Constant for the escaped escaping character.*/
+    private static final String DOUBLE_ESC = ESCAPE + ESCAPE;
+
     /** Constant for the radix of hex numbers.*/
     private static final int HEX_RADIX = 16;
 
@@ -210,6 +222,9 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
     /** Stores the layout object.*/
     private PropertiesConfigurationLayout layout;
 
+    /** The IOFactory for creating readers and writers.*/
+    private volatile IOFactory ioFactory;
+
     /** Allow file inclusion or not */
     private boolean includesAllowed;
 
@@ -401,6 +416,43 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
         return new PropertiesConfigurationLayout(this);
     }
 
+    /**
+     * Returns the <code>IOFactory</code> to be used for creating readers and
+     * writers when loading or saving this configuration.
+     *
+     * @return the <code>IOFactory</code>
+     * @since 1.7
+     */
+    public IOFactory getIOFactory()
+    {
+        return (ioFactory != null) ? ioFactory : DEFAULT_IO_FACTORY;
+    }
+
+    /**
+     * Sets the <code>IOFactory</code> to be used for creating readers and
+     * writers when loading or saving this configuration. Using this method a
+     * client can customize the reader and writer classes used by the load and
+     * save operations. Note that this method must be called before invoking
+     * one of the <code>load()</code> and <code>save()</code> methods.
+     * Especially, if you want to use a custom <code>IOFactory</code> for
+     * changing the <code>PropertiesReader</code>, you cannot load the
+     * configuration data in the constructor.
+     *
+     * @param ioFactory the new <code>IOFactory</code> (must not be <b>null</b>)
+     * @throws IllegalArgumentException if the <code>IOFactory</code> is
+     *         <b>null</b>
+     * @since 1.7
+     */
+    public void setIOFactory(IOFactory ioFactory)
+    {
+        if (ioFactory == null)
+        {
+            throw new IllegalArgumentException("IOFactory must not be null!");
+        }
+
+        this.ioFactory = ioFactory;
+    }
+
     /**
      * Load the properties from the given reader.
      * Note that the <code>clear()</code> method is not called, so
@@ -539,6 +591,24 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
         return s.length() < 1 || COMMENT_CHARS.indexOf(s.charAt(0)) >= 0;
     }
 
+    /**
+     * Returns the number of trailing backslashes. This is sometimes needed for
+     * the correct handling of escape characters.
+     *
+     * @param line the string to investigate
+     * @return the number of trailing backslashes
+     */
+    private static int countTrailingBS(String line)
+    {
+        int bsCount = 0;
+        for (int idx = line.length() - 1; idx >= 0 && line.charAt(idx) == '\\'; idx--)
+        {
+            bsCount++;
+        }
+
+        return bsCount;
+    }
+
     /**
      * This class is used to read properties lines. These lines do
      * not terminate with new-line chars but rather when there is no
@@ -556,6 +626,9 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
         /** Stores the value of the last read property.*/
         private String propertyValue;
 
+        /** Stores the property separator of the last read property.*/
+        private String propertySeparator = DEFAULT_SEPARATOR;
+
         /** Stores the list delimiter character.*/
         private char delimiter;
 
@@ -571,7 +644,7 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
 
         /**
          * Creates a new instance of <code>PropertiesReader</code> and sets
-         * the underlaying reader and the list delimiter.
+         * the underlying reader and the list delimiter.
          *
          * @param reader the reader
          * @param listDelimiter the list delimiter character
@@ -652,9 +725,7 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
             }
 
             // parse the line
-            String[] property = parseProperty(line);
-            propertyName = StringEscapeUtils.unescapeJava(property[0]);
-            propertyValue = unescapeJava(property[1], delimiter);
+            parseProperty(line);
             return true;
         }
 
@@ -696,6 +767,79 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
             return propertyValue;
         }
 
+        /**
+         * Returns the separator that was used for the last read property. The
+         * separator can be stored so that it can later be restored when saving
+         * the configuration.
+         *
+         * @return the separator for the last read property
+         * @since 1.7
+         */
+        public String getPropertySeparator()
+        {
+            return propertySeparator;
+        }
+
+        /**
+         * Parses a line read from the properties file. This method is called
+         * for each non-comment line read from the source file. Its task is to
+         * split the passed in line into the property key and its value. The
+         * results of the parse operation can be stored by calling the
+         * <code>initPropertyXXX()</code> methods.
+         *
+         * @param line the line read from the properties file
+         * @since 1.7
+         */
+        protected void parseProperty(String line)
+        {
+            String[] property = doParseProperty(line);
+            initPropertyName(property[0]);
+            initPropertyValue(property[1]);
+            initPropertySeparator(property[2]);
+        }
+
+        /**
+         * Sets the name of the current property. This method can be called by
+         * <code>parseProperty()</code> for storing the results of the parse
+         * operation. It also ensures that the property key is correctly
+         * escaped.
+         *
+         * @param name the name of the current property
+         * @since 1.7
+         */
+        protected void initPropertyName(String name)
+        {
+            propertyName = StringEscapeUtils.unescapeJava(name);
+        }
+
+        /**
+         * Sets the value of the current property. This method can be called by
+         * <code>parseProperty()</code> for storing the results of the parse
+         * operation. It also ensures that the property value is correctly
+         * escaped.
+         *
+         * @param value the value of the current property
+         * @since 1.7
+         */
+        protected void initPropertyValue(String value)
+        {
+            propertyValue = unescapeJava(value, delimiter);
+        }
+
+        /**
+         * Sets the separator of the current property. This method can be called
+         * by <code>parseProperty()</code>. It allows the associated layout
+         * object to keep track of the property separators. When saving the
+         * configuration the separators can be restored.
+         *
+         * @param value the separator used for the current property
+         * @since 1.7
+         */
+        protected void initPropertySeparator(String value)
+        {
+            propertySeparator = value;
+        }
+
         /**
          * Checks if the passed in line should be combined with the following.
          * This is true, if the line ends with an odd number of backslashes.
@@ -705,30 +849,24 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
          */
         private static boolean checkCombineLines(String line)
         {
-            int bsCount = 0;
-            for (int idx = line.length() - 1; idx >= 0 && line.charAt(idx) == '\\'; idx--)
-            {
-                bsCount++;
-            }
-
-            return bsCount % 2 != 0;
+            return countTrailingBS(line) % 2 != 0;
         }
 
         /**
-         * Parse a property line and return the key and the value in an array.
+         * Parse a property line and return the key, the value, and the separator in an array.
          *
          * @param line the line to parse
-         * @return an array with the property's key and value
-         * @since 1.2
+         * @return an array with the property's key, value, and separator
          */
-        private static String[] parseProperty(String line)
+        private static String[] doParseProperty(String line)
         {
             // sorry for this spaghetti code, please replace it as soon as
             // possible with a regexp when the Java 1.3 requirement is dropped
 
-            String[] result = new String[2];
+            String[] result = new String[3];
             StringBuffer key = new StringBuffer();
             StringBuffer value = new StringBuffer();
+            StringBuffer separator = new StringBuffer();
 
             // state of the automaton:
             // 0: key parsing
@@ -751,11 +889,13 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
                         else if (ArrayUtils.contains(WHITE_SPACE, c))
                         {
                             // switch to the separator crossing state
+                            separator.append(c);
                             state = 2;
                         }
                         else if (ArrayUtils.contains(SEPARATORS, c))
                         {
                             // switch to the value parsing state
+                            separator.append(c);
                             state = 3;
                         }
                         else
@@ -784,19 +924,14 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
                         break;
 
                     case 2:
-                        if (ArrayUtils.contains(WHITE_SPACE, c))
+                        if (ArrayUtils.contains(WHITE_SPACE, c) || ArrayUtils.contains(SEPARATORS, c))
                         {
-                            // do nothing, eat all white spaces
-                            state = 2;
-                        }
-                        else if (ArrayUtils.contains(SEPARATORS, c))
-                        {
-                            // switch to the value parsing state
-                            state = 3;
+                            // record the separator
+                            separator.append(c);
                         }
                         else
                         {
-                            // any other character indicates we encoutered the beginning of the value
+                            // any other character indicates we encountered the beginning of the value
                             value.append(c);
 
                             // switch to the value parsing state
@@ -813,19 +948,34 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
 
             result[0] = key.toString().trim();
             result[1] = value.toString().trim();
+            result[2] = separator.toString();
 
             return result;
         }
     } // class PropertiesReader
 
     /**
-     * This class is used to write properties lines.
+     * This class is used to write properties lines. The most important method
+     * is <code>writeProperty(String, Object, boolean)</code>, which is called
+     * during a save operation for each property found in the configuration.
      */
     public static class PropertiesWriter extends FilterWriter
     {
+        /** Constant for the initial size when creating a string buffer. */
+        private static final int BUF_SIZE = 8;
+
         /** The delimiter for multi-valued properties.*/
         private char delimiter;
 
+        /** The separator to be used for the current property. */
+        private String currentSeparator;
+
+        /** The global separator. If set, it overrides the current separator.*/
+        private String globalSeparator;
+
+        /** The line separator.*/
+        private String lineSeparator;
+
         /**
          * Constructor.
          *
@@ -838,6 +988,79 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
             this.delimiter = delimiter;
         }
 
+        /**
+         * Returns the current property separator.
+         *
+         * @return the current property separator
+         * @since 1.7
+         */
+        public String getCurrentSeparator()
+        {
+            return currentSeparator;
+        }
+
+        /**
+         * Sets the current property separator. This separator is used when
+         * writing the next property.
+         *
+         * @param currentSeparator the current property separator
+         * @since 1.7
+         */
+        public void setCurrentSeparator(String currentSeparator)
+        {
+            this.currentSeparator = currentSeparator;
+        }
+
+        /**
+         * Returns the global property separator.
+         *
+         * @return the global property separator
+         * @since 1.7
+         */
+        public String getGlobalSeparator()
+        {
+            return globalSeparator;
+        }
+
+        /**
+         * Sets the global property separator. This separator corresponds to the
+         * <code>globalSeparator</code> property of
+         * {@link PropertiesConfigurationLayout}. It defines the separator to be
+         * used for all properties. If it is undefined, the current separator is
+         * used.
+         *
+         * @param globalSeparator the global property separator
+         * @since 1.7
+         */
+        public void setGlobalSeparator(String globalSeparator)
+        {
+            this.globalSeparator = globalSeparator;
+        }
+
+        /**
+         * Returns the line separator.
+         *
+         * @return the line separator
+         * @since 1.7
+         */
+        public String getLineSeparator()
+        {
+            return (lineSeparator != null) ? lineSeparator : LINE_SEPARATOR;
+        }
+
+        /**
+         * Sets the line separator. Each line written by this writer is
+         * terminated with this separator. If not set, the platform-specific
+         * line separator is used.
+         *
+         * @param lineSeparator the line separator to be used
+         * @since 1.7
+         */
+        public void setLineSeparator(String lineSeparator)
+        {
+            this.lineSeparator = lineSeparator;
+        }
+
         /**
          * Write a property.
          *
@@ -899,11 +1122,11 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
             }
             else
             {
-                v = escapeValue(value);
+                v = escapeValue(value, false);
             }
 
             write(escapeKey(key));
-            write(" = ");
+            write(fetchSeparator(key, value));
             write(v);
 
             writeln(null);
@@ -955,12 +1178,13 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
          * will be escaped.
          *
          * @param value the property value
+         * @param inList a flag whether the value is part of a list
          * @return the escaped property value
          * @since 1.3
          */
-        private String escapeValue(Object value)
+        private String escapeValue(Object value, boolean inList)
         {
-            String escapedValue = StringEscapeUtils.escapeJava(String.valueOf(value));
+            String escapedValue = handleBackslashs(value, inList);
             if (delimiter != 0)
             {
                 escapedValue = StringUtils.replace(escapedValue, String.valueOf(delimiter), ESCAPE + delimiter);
@@ -968,6 +1192,46 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
             return escapedValue;
         }
 
+        /**
+         * Performs the escaping of backslashes in the specified properties
+         * value. Because a double backslash is used to escape the escape
+         * character of a list delimiter, double backslashes also have to be
+         * escaped if the property is part of a (single line) list. Then, in all
+         * cases each backslash has to be doubled in order to produce a valid
+         * properties file.
+         *
+         * @param value the value to be escaped
+         * @param inList a flag whether the value is part of a list
+         * @return the value with escaped backslashes as string
+         */
+        private String handleBackslashs(Object value, boolean inList)
+        {
+            String strValue = String.valueOf(value);
+
+            if (inList && strValue.indexOf(DOUBLE_ESC) >= 0)
+            {
+                char esc = ESCAPE.charAt(0);
+                StringBuffer buf = new StringBuffer(strValue.length() + BUF_SIZE);
+                for (int i = 0; i < strValue.length(); i++)
+                {
+                    if (strValue.charAt(i) == esc && i < strValue.length() - 1
+                            && strValue.charAt(i + 1) == esc)
+                    {
+                        buf.append(DOUBLE_ESC).append(DOUBLE_ESC);
+                        i++;
+                    }
+                    else
+                    {
+                        buf.append(strValue.charAt(i));
+                    }
+                }
+
+                strValue = buf.toString();
+            }
+
+            return StringEscapeUtils.escapeJava(strValue);
+        }
+
         /**
          * Transforms a list of values into a single line value.
          *
@@ -980,19 +1244,19 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
             if (!values.isEmpty())
             {
                 Iterator it = values.iterator();
-                String lastValue = escapeValue(it.next());
+                String lastValue = escapeValue(it.next(), true);
                 StringBuffer buf = new StringBuffer(lastValue);
                 while (it.hasNext())
                 {
                     // if the last value ended with an escape character, it has
                     // to be escaped itself; otherwise the list delimiter will
                     // be escaped
-                    if (lastValue.endsWith(ESCAPE))
+                    if (lastValue.endsWith(ESCAPE) && (countTrailingBS(lastValue) / 2) % 2 != 0)
                     {
                         buf.append(ESCAPE).append(ESCAPE);
                     }
                     buf.append(delimiter);
-                    lastValue = escapeValue(it.next());
+                    lastValue = escapeValue(it.next(), true);
                     buf.append(lastValue);
                 }
                 return buf.toString();
@@ -1017,11 +1281,109 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
             {
                 write(s);
             }
-            write(LINE_SEPARATOR);
+            write(getLineSeparator());
         }
 
+        /**
+         * Returns the separator to be used for the given property. This method
+         * is called by <code>writeProperty()</code>. The string returned here
+         * is used as separator between the property key and its value. Per
+         * default the method checks whether a global separator is set. If this
+         * is the case, it is returned. Otherwise the separator returned by
+         * <code>getCurrentSeparator()</code> is used, which was set by the
+         * associated layout object. Derived classes may implement a different
+         * strategy for defining the separator.
+         *
+         * @param key the property key
+         * @param value the value
+         * @return the separator to be used
+         * @since 1.7
+         */
+        protected String fetchSeparator(String key, Object value)
+        {
+            return (getGlobalSeparator() != null) ? getGlobalSeparator()
+                    : getCurrentSeparator();
+        }
     } // class PropertiesWriter
 
+    /**
+     * <p>
+     * Definition of an interface that allows customization of read and write
+     * operations.
+     * </p>
+     * <p>
+     * For reading and writing properties files the inner classes
+     * <code>PropertiesReader</code> and <code>PropertiesWriter</code> are used.
+     * This interface defines factory methods for creating both a
+     * <code>PropertiesReader</code> and a <code>PropertiesWriter</code>. An
+     * object implementing this interface can be passed to the
+     * <code>setIOFactory()</code> method of
+     * <code>PropertiesConfiguration</code>. Every time the configuration is
+     * read or written the <code>IOFactory</code> is asked to create the
+     * appropriate reader or writer object. This provides an opportunity to
+     * inject custom reader or writer implementations.
+     * </p>
+     *
+     * @since 1.7
+     */
+    public interface IOFactory
+    {
+        /**
+         * Creates a <code>PropertiesReader</code> for reading a properties
+         * file. This method is called whenever the
+         * <code>PropertiesConfiguration</code> is loaded. The reader returned
+         * by this method is then used for parsing the properties file.
+         *
+         * @param in the underlying reader (of the properties file)
+         * @param delimiter the delimiter character for list parsing
+         * @return the <code>PropertiesReader</code> for loading the
+         *         configuration
+         */
+        PropertiesReader createPropertiesReader(Reader in, char delimiter);
+
+        /**
+         * Creates a <code>PropertiesWriter</code> for writing a properties
+         * file. This method is called before the
+         * <code>PropertiesConfiguration</code> is saved. The writer returned by
+         * this method is then used for writing the properties file.
+         *
+         * @param out the underlying writer (to the properties file)
+         * @param delimiter the delimiter character for list parsing
+         * @return the <code>PropertiesWriter</code> for saving the
+         *         configuration
+         */
+        PropertiesWriter createPropertiesWriter(Writer out, char delimiter);
+    }
+
+    /**
+     * <p>
+     * A default implementation of the <code>IOFactory</code> interface.
+     * </p>
+     * <p>
+     * This class implements the <code>createXXXX()</code> methods defined by
+     * the <code>IOFactory</code> interface in a way that the default objects
+     * (i.e. <code>PropertiesReader</code> and <code>PropertiesWriter</code> are
+     * returned. Customizing either the reader or the writer (or both) can be
+     * done by extending this class and overriding the corresponding
+     * <code>createXXXX()</code> method.
+     * </p>
+     *
+     * @since 1.7
+     */
+    public static class DefaultIOFactory implements IOFactory
+    {
+        public PropertiesReader createPropertiesReader(Reader in, char delimiter)
+        {
+            return new PropertiesReader(in, delimiter);
+        }
+
+        public PropertiesWriter createPropertiesWriter(Writer out,
+                char delimiter)
+        {
+            return new PropertiesWriter(out, delimiter);
+        }
+    }
+
     /**
      * <p>Unescapes any Java literals found in the <code>String</code> to a
      * <code>Writer</code>.</p> This is a slightly modified version of the
@@ -1156,13 +1518,13 @@ public class PropertiesConfiguration extends AbstractFileConfiguration
      */
     private void loadIncludeFile(String fileName) throws ConfigurationException
     {
-        URL url = ConfigurationUtils.locate(getBasePath(), fileName);
+        URL url = ConfigurationUtils.locate(getFileSystem(), getBasePath(), fileName);
         if (url == null)
         {
             URL baseURL = getURL();
             if (baseURL != null)
             {
-                url = ConfigurationUtils.locate(baseURL.toString(), fileName);
+                url = ConfigurationUtils.locate(getFileSystem(), baseURL.toString(), fileName);
             }
         }
 
