@@ -110,15 +110,15 @@ import org.apache.commons.lang.StringUtils;
  * </p>
  *
  * @author <a
- * href="http://jakarta.apache.org/commons/configuration/team-list.html">Commons
+ * href="http://commons.apache.org/configuration/team-list.html">Commons
  * Configuration team</a>
- * @version $Id: PropertiesConfigurationLayout.java 439648 2006-09-02 20:42:10Z oheger $
+ * @version $Id: PropertiesConfigurationLayout.java 759750 2009-03-29 19:15:36Z oheger $
  * @since 1.3
  */
 public class PropertiesConfigurationLayout implements ConfigurationListener
 {
     /** Constant for the line break character. */
-    private static final String CR = System.getProperty("line.separator");
+    private static final String CR = "\n";
 
     /** Constant for the default comment prefix. */
     private static final String COMMENT_PREFIX = "# ";
@@ -132,6 +132,12 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
     /** Stores the header comment. */
     private String headerComment;
 
+    /** The global separator that will be used for all properties. */
+    private String globalSeparator;
+
+    /** The line separator.*/
+    private String lineSeparator;
+
     /** A counter for determining nested load calls. */
     private int loadCounter;
 
@@ -186,12 +192,13 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
     }
 
     /**
-     * Returns the comment for the specified property key in a cononical form.
+     * Returns the comment for the specified property key in a canonical form.
      * &quot;Canonical&quot; means that either all lines start with a comment
-     * character or none. The <code>commentChar</code> parameter is <b>false</b>,
+     * character or none. If the <code>commentChar</code> parameter is <b>false</b>,
      * all comment characters are removed, so that the result is only the plain
      * text of the comment. Otherwise it is ensured that each line of the
-     * comment starts with a comment character.
+     * comment starts with a comment character. Also, line breaks in the comment
+     * are normalized to the line separator &quot;\n&quot;.
      *
      * @param key the key of the property
      * @param commentChar determines whether all lines should start with comment
@@ -348,7 +355,8 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
      * Sets the &quot;force single line&quot; flag. If this flag is set, all
      * properties with multiple values are written on single lines. This mode
      * provides more compatibility with <code>java.lang.Properties</code>,
-     * which cannot deal with multiple definitions of a single property.
+     * which cannot deal with multiple definitions of a single property. This
+     * mode has no effect if the list delimiter parsing is disabled.
      *
      * @param f the force single line flag
      */
@@ -357,6 +365,93 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
         forceSingleLine = f;
     }
 
+    /**
+     * Returns the separator for the property with the given key.
+     *
+     * @param key the property key
+     * @return the property separator for this property
+     * @since 1.7
+     */
+    public String getSeparator(String key)
+    {
+        return fetchLayoutData(key).getSeparator();
+    }
+
+    /**
+     * Sets the separator to be used for the property with the given key. The
+     * separator is the string between the property key and its value. For new
+     * properties &quot; = &quot; is used. When a properties file is read, the
+     * layout tries to determine the separator for each property. With this
+     * method the separator can be changed. To be compatible with the properties
+     * format only the characters <code>=</code> and <code>:</code> (with or
+     * without whitespace) should be used, but this method does not enforce this
+     * - it accepts arbitrary strings. If the key refers to a property with
+     * multiple values that are written on multiple lines, this separator will
+     * be used on all lines.
+     *
+     * @param key the key for the property
+     * @param sep the separator to be used for this property
+     * @since 1.7
+     */
+    public void setSeparator(String key, String sep)
+    {
+        fetchLayoutData(key).setSeparator(sep);
+    }
+
+    /**
+     * Returns the global separator.
+     *
+     * @return the global properties separator
+     * @since 1.7
+     */
+    public String getGlobalSeparator()
+    {
+        return globalSeparator;
+    }
+
+    /**
+     * Sets the global separator for properties. With this method a separator
+     * can be set that will be used for all properties when writing the
+     * configuration. This is an easy way of determining the properties
+     * separator globally. To be compatible with the properties format only the
+     * characters <code>=</code> and <code>:</code> (with or without whitespace)
+     * should be used, but this method does not enforce this - it accepts
+     * arbitrary strings. If the global separator is set to <b>null</b>,
+     * property separators are not changed. This is the default behavior as it
+     * produces results that are closer to the original properties file.
+     *
+     * @param globalSeparator the separator to be used for all properties
+     * @since 1.7
+     */
+    public void setGlobalSeparator(String globalSeparator)
+    {
+        this.globalSeparator = globalSeparator;
+    }
+
+    /**
+     * Returns the line separator.
+     *
+     * @return the line separator
+     * @since 1.7
+     */
+    public String getLineSeparator()
+    {
+        return lineSeparator;
+    }
+
+    /**
+     * Sets the line separator. When writing the properties configuration, all
+     * lines are terminated with this separator. If no separator was set, the
+     * platform-specific default line separator is used.
+     *
+     * @param lineSeparator the line separator
+     * @since 1.7
+     */
+    public void setLineSeparator(String lineSeparator)
+    {
+        this.lineSeparator = lineSeparator;
+    }
+
     /**
      * Returns a set with all property keys managed by this object.
      *
@@ -380,8 +475,9 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
         {
             getConfiguration().removeConfigurationListener(this);
         }
-        PropertiesConfiguration.PropertiesReader reader = new PropertiesConfiguration.PropertiesReader(
-                in, getConfiguration().getListDelimiter());
+        PropertiesConfiguration.PropertiesReader reader = getConfiguration()
+                .getIOFactory().createPropertiesReader(in,
+                        getConfiguration().getListDelimiter());
 
         try
         {
@@ -414,6 +510,7 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
                     {
                         data.setComment(comment);
                         data.setBlancLines(blancLines);
+                        data.setSeparator(reader.getPropertySeparator());
                     }
                 }
             }
@@ -442,11 +539,19 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
     {
         try
         {
-            PropertiesConfiguration.PropertiesWriter writer = new PropertiesConfiguration.PropertiesWriter(
-                    out, getConfiguration().getListDelimiter());
+            char delimiter = getConfiguration().isDelimiterParsingDisabled() ? 0
+                    : getConfiguration().getListDelimiter();
+            PropertiesConfiguration.PropertiesWriter writer = getConfiguration()
+                    .getIOFactory().createPropertiesWriter(out, delimiter);
+            writer.setGlobalSeparator(getGlobalSeparator());
+            if (getLineSeparator() != null)
+            {
+                writer.setLineSeparator(getLineSeparator());
+            }
+
             if (headerComment != null)
             {
-                writer.writeln(getCanonicalHeaderComment(true));
+                writeComment(writer, getCanonicalHeaderComment(true));
                 writer.writeln(null);
             }
 
@@ -456,21 +561,21 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
                 if (getConfiguration().containsKey(key))
                 {
 
-                    // Output blanc lines before property
+                    // Output blank lines before property
                     for (int i = 0; i < getBlancLinesBefore(key); i++)
                     {
                         writer.writeln(null);
                     }
 
                     // Output the comment
-                    if (getComment(key) != null)
-                    {
-                        writer.writeln(getCanonicalComment(key, true));
-                    }
+                    writeComment(writer, getCanonicalComment(key, true));
 
                     // Output the property and its value
+                    boolean singleLine = (isForceSingleLine() || isSingleLine(key))
+                            && !getConfiguration().isDelimiterParsingDisabled();
+                    writer.setCurrentSeparator(getSeparator(key));
                     writer.writeProperty(key, getConfiguration().getProperty(
-                            key), isForceSingleLine() || isSingleLine(key));
+                            key), singleLine);
                 }
             }
             writer.flush();
@@ -679,7 +784,7 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
      * Checks if parts of the passed in comment can be used as header comment.
      * This method checks whether a header comment can be defined (i.e. whether
      * this is the first comment in the loaded file). If this is the case, it is
-     * searched for the lates blanc line. This line will mark the end of the
+     * searched for the latest blanc line. This line will mark the end of the
      * header comment. The return value is the index of the first line in the
      * passed in list, which does not belong to the header comment.
      *
@@ -723,6 +828,25 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
         }
     }
 
+    /**
+     * Helper method for writing a comment line. This method ensures that the
+     * correct line separator is used if the comment spans multiple lines.
+     *
+     * @param writer the writer
+     * @param comment the comment to write
+     * @throws IOException if an IO error occurs
+     */
+    private static void writeComment(
+            PropertiesConfiguration.PropertiesWriter writer, String comment)
+            throws IOException
+    {
+        if (comment != null)
+        {
+            writer.writeln(StringUtils.replace(comment, CR, writer
+                    .getLineSeparator()));
+        }
+    }
+
     /**
      * A helper class for storing all layout related information for a
      * configuration property.
@@ -732,6 +856,9 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
         /** Stores the comment for the property. */
         private StringBuffer comment;
 
+        /** The separator to be used for this property. */
+        private String separator;
+
         /** Stores the number of blanc lines before this property. */
         private int blancLines;
 
@@ -744,6 +871,7 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
         public PropertyLayoutData()
         {
             singleLine = true;
+            separator = PropertiesConfiguration.DEFAULT_SEPARATOR;
         }
 
         /**
@@ -835,6 +963,26 @@ public class PropertiesConfigurationLayout implements ConfigurationListener
             return (comment == null) ? null : comment.toString();
         }
 
+        /**
+         * Returns the separator that was used for this property.
+         *
+         * @return the property separator
+         */
+        public String getSeparator()
+        {
+            return separator;
+        }
+
+        /**
+         * Sets the separator to be used for the represented property.
+         *
+         * @param separator the property separator
+         */
+        public void setSeparator(String separator)
+        {
+            this.separator = separator;
+        }
+
         /**
          * Creates a copy of this object.
          *
