@@ -49,7 +49,7 @@ import org.apache.commons.lang.SystemUtils;
  * A utility class to convert the configuration properties into any type.
  *
  * @author Emmanuel Bourg
- * @version $Revision: 589380 $, $Date: 2007-10-28 17:37:35 +0100 (So, 28 Okt 2007) $
+ * @version $Revision: 1034102 $, $Date: 2010-11-11 21:28:05 +0100 (Do, 11. Nov 2010) $
  * @since 1.1
  */
 public final class PropertyConverter
@@ -499,9 +499,10 @@ public final class PropertyConverter
      *
      * @param s          the string to split
      * @param delimiter  the delimiter
+     * @param trim       a flag whether the single elements should be trimmed
      * @return a list with the single tokens
      */
-    public static List split(String s, char delimiter)
+    public static List split(String s, char delimiter, boolean trim)
     {
         if (s == null)
         {
@@ -534,8 +535,13 @@ public final class PropertyConverter
             {
                 if (c == delimiter)
                 {
-                    // found a list delimiter -> add token and reset buffer
-                    list.add(token.toString().trim());
+                    // found a list delimiter -> add token and resetDefaultFileSystem buffer
+                    String t = token.toString();
+                    if (trim)
+                    {
+                        t = t.trim();
+                    }
+                    list.add(t);
                     token = new StringBuffer();
                 }
                 else if (c == LIST_ESC_CHAR)
@@ -558,26 +564,60 @@ public final class PropertyConverter
             token.append(LIST_ESC_CHAR);
         }
         // Add last token
-        list.add(token.toString().trim());
+        String t = token.toString();
+        if (trim)
+        {
+            t = t.trim();
+        }
+        list.add(t);
 
         return list;
     }
 
+    /**
+     * Split a string on the specified delimiter always trimming the elements.
+     * This is a shortcut for <code>split(s, delimiter, true)</code>.
+     *
+     * @param s          the string to split
+     * @param delimiter  the delimiter
+     * @return a list with the single tokens
+     */
+    public static List split(String s, char delimiter)
+    {
+        return split(s, delimiter, true);
+    }
+
     /**
      * Escapes the delimiters that might be contained in the given string. This
-     * method ensures that list delimiter characters that are part of a
-     * property's value are correctly escaped when a configuration is saved to a
-     * file. Otherwise when loaded again the property will be treated as a list
-     * property. A single backslash will also be escaped.
+     * method works like {@link #escapeListDelimiter(String, char)}. In addition,
+     * a single backslash will also be escaped.
      *
      * @param s the string with the value
      * @param delimiter the list delimiter to use
-     * @return the correctly esaped string
+     * @return the correctly escaped string
      */
     public static String escapeDelimiters(String s, char delimiter)
     {
         String s1 = StringUtils.replace(s, LIST_ESCAPE, LIST_ESCAPE + LIST_ESCAPE);
-        return StringUtils.replace(s1, String.valueOf(delimiter), LIST_ESCAPE + delimiter);
+        return escapeListDelimiter(s1, delimiter);
+    }
+
+    /**
+     * Escapes the list delimiter if it is contained in the given string. This
+     * method ensures that list delimiter characters that are part of a
+     * property's value are correctly escaped when a configuration is saved to a
+     * file. Otherwise when loaded again the property will be treated as a list
+     * property.
+     *
+     * @param s the string with the value
+     * @param delimiter the list delimiter to use
+     * @return the escaped string
+     * @since 1.7
+     */
+    public static String escapeListDelimiter(String s, char delimiter)
+    {
+        return StringUtils.replace(s, String.valueOf(delimiter), LIST_ESCAPE
+                + delimiter);
     }
 
     /**
