@@ -16,10 +16,13 @@
  */
 package org.apache.commons.configuration.beanutils;
 
+import java.beans.PropertyDescriptor;
 import java.lang.reflect.InvocationTargetException;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
@@ -51,9 +54,9 @@ import org.apache.commons.lang.ClassUtils;
  *
  * @since 1.3
  * @author Oliver Heger
- * @version $Id: BeanHelper.java 570462 2007-08-28 15:56:49Z oheger $
+ * @version $Id: BeanHelper.java 1089793 2011-04-07 09:45:51Z oheger $
  */
-public class BeanHelper
+public final class BeanHelper
 {
     /** Stores a map with the registered bean factories. */
     private static Map beanFactories = Collections.synchronizedMap(new HashMap());
@@ -158,25 +161,90 @@ public class BeanHelper
     public static void initBean(Object bean, BeanDeclaration data)
             throws ConfigurationRuntimeException
     {
-        Map properties = data.getBeanProperties();
-        if (properties != null)
-        {
-            for (Iterator it = properties.keySet().iterator(); it.hasNext();)
-            {
-                String propName = (String) it.next();
-                initProperty(bean, propName, properties.get(propName));
-            }
-        }
+        initBeanProperties(bean, data);
 
         Map nestedBeans = data.getNestedBeanDeclarations();
         if (nestedBeans != null)
         {
-            for (Iterator it = nestedBeans.keySet().iterator(); it.hasNext();)
+            if (bean instanceof Collection)
             {
-                String propName = (String) it.next();
-                initProperty(bean, propName, createBean(
-                        (BeanDeclaration) nestedBeans.get(propName), null));
+                Collection coll = (Collection) bean;
+                if (nestedBeans.size() == 1)
+                {
+                    Map.Entry e = (Map.Entry) nestedBeans.entrySet().iterator().next();
+                    String propName = (String) e.getKey();
+                    Class defaultClass = getDefaultClass(bean, propName);
+                    if (e.getValue() instanceof List)
+                    {
+                        Iterator iter = ((List) e.getValue()).iterator();
+                        while (iter.hasNext())
+                        {
+                            coll.add(createBean((BeanDeclaration) iter.next(), defaultClass));
+                        }
+                    }
+                    else
+                    {
+                        BeanDeclaration decl = (BeanDeclaration) e.getValue();
+                        coll.add(createBean(decl, defaultClass));
+                    }
+                }
             }
+            else
+            {
+                for (Iterator it = nestedBeans.entrySet().iterator(); it.hasNext();)
+                {
+                    Map.Entry e = (Map.Entry) it.next();
+                    String propName = (String) e.getKey();
+                    Class defaultClass = getDefaultClass(bean, propName);
+                    initProperty(bean, propName, createBean(
+                        (BeanDeclaration) e.getValue(), defaultClass));
+                }
+            }
+        }
+    }
+
+    /**
+     * Initializes the beans properties.
+     *
+     * @param bean the bean to be initialized
+     * @param data the bean declaration
+     * @throws ConfigurationRuntimeException if a property cannot be set
+     */
+    public static void initBeanProperties(Object bean, BeanDeclaration data)
+            throws ConfigurationRuntimeException
+    {
+        Map properties = data.getBeanProperties();
+        if (properties != null)
+        {
+            for (Iterator it = properties.entrySet().iterator(); it.hasNext();)
+            {
+                Map.Entry e = (Map.Entry) it.next();
+                String propName = (String) e.getKey();
+                initProperty(bean, propName, e.getValue());
+            }
+        }
+    }
+
+    /**
+     * Return the Class of the property if it can be determined.
+     * @param bean The bean containing the property.
+     * @param propName The name of the property.
+     * @return The class associated with the property or null.
+     */
+    private static Class getDefaultClass(Object bean, String propName)
+    {
+        try
+        {
+            PropertyDescriptor desc = PropertyUtils.getPropertyDescriptor(bean, propName);
+            if (desc == null)
+            {
+                return null;
+            }
+            return desc.getPropertyType();
+        }
+        catch (Exception ex)
+        {
+            return null;
         }
     }
 
@@ -195,7 +263,7 @@ public class BeanHelper
         if (!PropertyUtils.isWriteable(bean, propName))
         {
             throw new ConfigurationRuntimeException("Property " + propName
-                    + " cannot be set!");
+                    + " cannot be set on " + bean.getClass().getName());
         }
 
         try
@@ -212,6 +280,23 @@ public class BeanHelper
         }
     }
 
+    /**
+     * Set a property on the bean only if the property exists
+     *
+     * @param bean the bean
+     * @param propName the name of the property
+     * @param value the property's value
+     * @throws ConfigurationRuntimeException if the property is not writeable or
+     *         an error occurred
+     */
+    public static void setProperty(Object bean, String propName, Object value)
+    {
+        if (PropertyUtils.isWriteable(bean, propName))
+        {
+            initProperty(bean, propName, value);
+        }
+    }
+
     /**
      * The main method for creating and initializing beans from a configuration.
      * This method will return an initialized instance of the bean class
@@ -283,9 +368,9 @@ public class BeanHelper
 
     /**
      * Returns a <code>java.lang.Class</code> object for the specified name.
-     * This method and the helper method it invokes are very similar to code
-     * extracted from the <code>ClassLoaderUtils</code> class of Commons
-     * Jelly. It should be replaced if Commons Lang provides a generic version.
+     * Because class loading can be tricky in some environments the code for
+     * retrieving a class by its name was extracted into this helper method. So
+     * if changes are necessary, they can be made at a single place.
      *
      * @param name the name of the class to be loaded
      * @param callingClass the calling class
