@@ -40,12 +40,12 @@ import org.apache.commons.lang.StringUtils;
 
 /**
  * NeXT / OpenStep style configuration. This configuration can read and write
- * ASCII plist files. It support the GNUStep extension to specify date objects.
+ * ASCII plist files. It supports the GNUStep extension to specify date objects.
  * <p>
  * References:
  * <ul>
  *   <li><a
- * href="http://developer.apple.com/documentation/Cocoa/Conceptual/PropertyLists/Articles/OldStylePListsConcept.html">
+ * href="http://developer.apple.com/documentation/Cocoa/Conceptual/PropertyLists/OldStylePlists/OldStylePLists.html">
  * Apple Documentation - Old-Style ASCII Property Lists</a></li>
  *   <li><a
  * href="http://www.gnustep.org/resources/documentation/Developer/Base/Reference/NSPropertyList.html">
@@ -78,7 +78,7 @@ import org.apache.commons.lang.StringUtils;
  * @since 1.2
  *
  * @author Emmanuel Bourg
- * @version $Revision: 590474 $, $Date: 2007-10-30 22:35:11 +0100 (Di, 30 Okt 2007) $
+ * @version $Revision: 797282 $, $Date: 2009-07-24 02:39:29 +0200 (Fr, 24. Jul 2009) $
  */
 public class PropertyListConfiguration extends AbstractHierarchicalFileConfiguration
 {
@@ -252,18 +252,7 @@ public class PropertyListConfiguration extends AbstractHierarchicalFileConfigura
             out.print(padding + quoteString(node.getName()) + " = ");
         }
 
-        // get all non trivial nodes
         List children = new ArrayList(node.getChildren());
-        Iterator it = children.iterator();
-        while (it.hasNext())
-        {
-            Node child = (Node) it.next();
-            if (child.getValue() == null && (child.getChildren() == null || child.getChildren().isEmpty()))
-            {
-                it.remove();
-            }
-        }
-
         if (!children.isEmpty())
         {
             // skip a line, except for the root dictionary
@@ -275,7 +264,7 @@ public class PropertyListConfiguration extends AbstractHierarchicalFileConfigura
             out.println(padding + "{");
 
             // display the children
-            it = children.iterator();
+            Iterator it = children.iterator();
             while (it.hasNext())
             {
                 Node child = (Node) it.next();
@@ -304,6 +293,17 @@ public class PropertyListConfiguration extends AbstractHierarchicalFileConfigura
                 out.println();
             }
         }
+        else if (node.getValue() == null)
+        {
+            out.println();
+            out.print(padding + "{ };");
+
+            // line feed if the dictionary is not in an array
+            if (node.getParentNode() != null)
+            {
+                out.println();
+            }
+        }
         else
         {
             // display the leaf value
