@@ -22,16 +22,20 @@ import java.io.PrintWriter;
 import java.io.Reader;
 import java.io.Writer;
 import java.math.BigDecimal;
+import java.math.BigInteger;
 import java.net.URL;
 import java.text.DateFormat;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Calendar;
+import java.util.Collection;
 import java.util.Date;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.TimeZone;
+
 import javax.xml.parsers.SAXParser;
 import javax.xml.parsers.SAXParserFactory;
 
@@ -43,7 +47,6 @@ import org.apache.commons.configuration.HierarchicalConfiguration;
 import org.apache.commons.configuration.MapConfiguration;
 import org.apache.commons.lang.StringEscapeUtils;
 import org.apache.commons.lang.StringUtils;
-
 import org.xml.sax.Attributes;
 import org.xml.sax.EntityResolver;
 import org.xml.sax.InputSource;
@@ -73,7 +76,7 @@ import org.xml.sax.helpers.DefaultHandler;
  *         &lt;true/>
  *
  *         &lt;key>date&lt;/key>
- *         &lt;date>2005-01-01T12:00:00-0700&lt;/date>
+ *         &lt;date>2005-01-01T12:00:00Z&lt;/date>
  *
  *         &lt;key>data&lt;/key>
  *         &lt;data>RHJhY28gRG9ybWllbnMgTnVucXVhbSBUaXRpbGxhbmR1cw==&lt;/data>
@@ -114,7 +117,7 @@ import org.xml.sax.helpers.DefaultHandler;
  * @since 1.2
  *
  * @author Emmanuel Bourg
- * @version $Revision: 532336 $, $Date: 2007-04-25 14:23:01 +0200 (Mi, 25 Apr 2007) $
+ * @version $Revision: 902596 $, $Date: 2010-01-24 17:28:55 +0100 (So, 24. Jan 2010) $
  */
 public class XMLPropertyListConfiguration extends AbstractHierarchicalFileConfiguration
 {
@@ -133,6 +136,7 @@ public class XMLPropertyListConfiguration extends AbstractHierarchicalFileConfig
      */
     public XMLPropertyListConfiguration()
     {
+        initRoot();
     }
 
     /**
@@ -221,6 +225,14 @@ public class XMLPropertyListConfiguration extends AbstractHierarchicalFileConfig
 
     public void load(Reader in) throws ConfigurationException
     {
+        // We have to make sure that the root node is actually a PListNode.
+        // If this object was not created using the standard constructor, the
+        // root node is a plain Node.
+        if (!(getRootNode() instanceof PListNode))
+        {
+            initRoot();
+        }
+
         // set up the DTD validation
         EntityResolver resolver = new EntityResolver()
         {
@@ -301,6 +313,10 @@ public class XMLPropertyListConfiguration extends AbstractHierarchicalFileConfig
 
             out.println(padding + "</dict>");
         }
+        else if (node.getValue() == null)
+        {
+            out.println(padding + "<dict/>");
+        }
         else
         {
             Object value = node.getValue();
@@ -317,7 +333,10 @@ public class XMLPropertyListConfiguration extends AbstractHierarchicalFileConfig
 
         if (value instanceof Date)
         {
-            out.println(padding + "<date>" + PListNode.format.format((Date) value) + "</date>");
+            synchronized (PListNode.format)
+            {
+                out.println(padding + "<date>" + PListNode.format.format((Date) value) + "</date>");
+            }
         }
         else if (value instanceof Calendar)
         {
@@ -394,16 +413,28 @@ public class XMLPropertyListConfiguration extends AbstractHierarchicalFileConfig
             String base64 = new String(Base64.encodeBase64((byte[]) value));
             out.println(padding + "<data>" + StringEscapeUtils.escapeXml(base64) + "</data>");
         }
-        else
+        else if (value != null)
         {
             out.println(padding + "<string>" + StringEscapeUtils.escapeXml(String.valueOf(value)) + "</string>");
         }
+        else
+        {
+            out.println(padding + "<string/>");
+        }
+    }
+
+    /**
+     * Helper method for initializing the configuration's root node.
+     */
+    private void initRoot()
+    {
+        setRootNode(new PListNode());
     }
 
     /**
      * SAX Handler to build the configuration nodes while the document is being parsed.
      */
-    private class XMLPropertyListHandler extends DefaultHandler
+    private static class XMLPropertyListHandler extends DefaultHandler
     {
         /** The buffer containing the text node being read */
         private StringBuffer buffer = new StringBuffer();
@@ -557,8 +588,15 @@ public class XMLPropertyListConfiguration extends AbstractHierarchicalFileConfig
          */
         private static final long serialVersionUID = -7614060264754798317L;
 
-        /** The standard format of dates in plist files. */
-        private static DateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
+        /** The MacOS format of dates in plist files. */
+        private static DateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
+        static
+        {
+            format.setTimeZone(TimeZone.getTimeZone("UTC"));
+        }
+
+        /** The GNUstep format of dates in plist files. */
+        private static DateFormat gnustepFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss Z");
 
         /**
          * Update the value of the node. If the existing value is null, it's
@@ -574,10 +612,10 @@ public class XMLPropertyListConfiguration extends AbstractHierarchicalFileConfig
             {
                 setValue(value);
             }
-            else if (getValue() instanceof List)
+            else if (getValue() instanceof Collection)
             {
-                List list = (List) getValue();
-                list.add(value);
+                Collection collection = (Collection) getValue();
+                collection.add(value);
             }
             else
             {
@@ -597,7 +635,22 @@ public class XMLPropertyListConfiguration extends AbstractHierarchicalFileConfig
         {
             try
             {
-                addValue(format.parse(value));
+                if (value.indexOf(' ') != -1)
+                {
+                    // parse the date using the GNUstep format
+                    synchronized (gnustepFormat)
+                    {
+                        addValue(gnustepFormat.parse(value));
+                    }
+                }
+                else
+                {
+                    // parse the date using the MacOS X format
+                    synchronized (format)
+                    {
+                        addValue(format.parse(value));
+                    }
+                }
             }
             catch (ParseException e)
             {
@@ -624,7 +677,7 @@ public class XMLPropertyListConfiguration extends AbstractHierarchicalFileConfig
          */
         public void addIntegerValue(String value)
         {
-            addValue(new Integer(value));
+            addValue(new BigInteger(value));
         }
 
         /**
@@ -634,7 +687,7 @@ public class XMLPropertyListConfiguration extends AbstractHierarchicalFileConfig
          */
         public void addRealValue(String value)
         {
-            addValue(new Double(value));
+            addValue(new BigDecimal(value));
         }
 
         /**
