@@ -47,7 +47,7 @@ import org.apache.commons.lang.StringUtils;
  *
  * @since 1.3
  * @author Oliver Heger
- * @version $Id: DefaultConfigurationKey.java 439648 2006-09-02 20:42:10Z oheger $
+ * @version $Id: DefaultConfigurationKey.java 1158891 2011-08-17 20:08:39Z oheger $
  */
 public class DefaultConfigurationKey
 {
@@ -823,20 +823,26 @@ public class DefaultConfigurationKey
         {
             boolean result = false;
 
-            int idx = key.lastIndexOf(getExpressionEngine().getIndexStart());
-            if (idx > 0)
+            try
             {
-                int endidx = key.indexOf(getExpressionEngine().getIndexEnd(),
-                        idx);
-
-                if (endidx > idx + 1)
+                int idx = key.lastIndexOf(getExpressionEngine().getIndexStart());
+                if (idx > 0)
                 {
-                    indexValue = Integer.parseInt(key
-                            .substring(idx + 1, endidx));
-                    current = key.substring(0, idx);
-                    result = true;
+                    int endidx = key.indexOf(getExpressionEngine().getIndexEnd(),
+                            idx);
+
+                    if (endidx > idx + 1)
+                    {
+                        indexValue = Integer.parseInt(key.substring(idx + 1, endidx));
+                        current = key.substring(0, idx);
+                        result = true;
+                    }
                 }
             }
+            catch (NumberFormatException nfe)
+            {
+                result = false;
+            }
 
             return result;
         }
