@@ -34,17 +34,17 @@ import org.apache.commons.lang.StringUtils;
  * that is able to evaluate XPATH expressions.
  * </p>
  * <p>
- * This class makes use of <a href="http://commons.apache.org/jxpath/">
- * Commons JXPath</a> for handling XPath expressions and mapping them to the
- * nodes of a hierarchical configuration. This makes the rich and powerfull
- * XPATH syntax available for accessing properties from a configuration object.
+ * This class makes use of <a href="http://commons.apache.org/jxpath/"> Commons
+ * JXPath</a> for handling XPath expressions and mapping them to the nodes of a
+ * hierarchical configuration. This makes the rich and powerful XPATH syntax
+ * available for accessing properties from a configuration object.
  * </p>
  * <p>
  * For selecting properties arbitrary XPATH expressions can be used, which
  * select single or multiple configuration nodes. The associated
- * <code>Configuration</code> instance will directly pass the specified
- * property keys into this engine. If a key is not syntactically correct, an
- * exception will be thrown.
+ * <code>Configuration</code> instance will directly pass the specified property
+ * keys into this engine. If a key is not syntactically correct, an exception
+ * will be thrown.
  * </p>
  * <p>
  * For adding new properties, this expression engine uses a specific syntax: the
@@ -56,7 +56,7 @@ import org.apache.commons.lang.StringUtils;
  * select exactly one node, otherwise an exception will be thrown.</li>
  * <li>The name of the new element(s) to be added below this parent node. Here
  * either a single node name or a complete path of nodes (separated by the
- * &quot;/&quot; character) can be specified.</li>
+ * &quot;/&quot; character or &quot;@&quot; for an attribute) can be specified.</li>
  * </ol>
  * Some examples for valid keys that can be passed into the configuration's
  * <code>addProperty()</code> method follow:
@@ -96,10 +96,52 @@ import org.apache.commons.lang.StringUtils;
  * <code>table</code>, <code>fields</code>, <code>field</code>, and
  * <code>name</code> will be added.
  * </p>
+ * <p>
+ *
+ * <pre>
+ * &quot;/tables table/fields/field@type&quot;
+ * </pre>
+ *
+ * </p>
+ * <p>
+ * This is similar to the last example, but in this case a complex path ending
+ * with an attribute is defined.
+ * </p>
+ * <p>
+ * <strong>Note:</strong> This extended syntax for adding properties only works
+ * with the <code>addProperty()</code> method. <code>setProperty()</code> does
+ * not support creating new nodes this way.
+ * </p>
+ * <p>
+ * From version 1.7 on, it is possible to use regular keys in calls to
+ * <code>addProperty()</code> (i.e. keys that do not have to contain a
+ * whitespace as delimiter). In this case the key is evaluated, and the biggest
+ * part pointing to an existing node is determined. The remaining part is then
+ * added as new path. As an example consider the key
+ *
+ * <pre>
+ * &quot;tables/table[last()]/fields/field/name&quot;
+ * </pre>
+ *
+ * If the key does not point to an existing node, the engine will check the
+ * paths <code>&quot;tables/table[last()]/fields/field&quot;</code>,
+ * <code>&quot;tables/table[last()]/fields&quot;</code>,
+ * <code>&quot;tables/table[last()]&quot;</code>, and so on, until a key is
+ * found which points to a node. Let's assume that the last key listed above can
+ * be resolved in this way. Then from this key the following key is derived:
+ * <code>&quot;tables/table[last()] fields/field/name&quot;</code> by appending
+ * the remaining part after a whitespace. This key can now be processed using
+ * the original algorithm. Keys of this form can also be used with the
+ * <code>setProperty()</code> method. However, it is still recommended to use
+ * the old format because it makes explicit at which position new nodes should
+ * be added. For keys without a whitespace delimiter there may be ambiguities.
+ * </p>
  *
  * @since 1.3
- * @author Oliver Heger
- * @version $Id: XPathExpressionEngine.java 561230 2007-07-31 04:17:09Z rahul $
+ * @author <a
+ *         href="http://commons.apache.org/configuration/team-list.html">Commons
+ *         Configuration team</a>
+ * @version $Id: XPathExpressionEngine.java 1152357 2011-07-29 19:56:01Z oheger $
  */
 public class XPathExpressionEngine implements ExpressionEngine
 {
@@ -113,6 +155,12 @@ public class XPathExpressionEngine implements ExpressionEngine
     private static final String NODE_PATH_DELIMITERS = PATH_DELIMITER
             + ATTR_DELIMITER;
 
+    /**
+     * Constant for a space which is used as delimiter in keys for adding
+     * properties.
+     */
+    private static final String SPACE = " ";
+
     /**
      * Executes a query. The passed in property key is directly passed to a
      * JXPath context.
@@ -138,13 +186,13 @@ public class XPathExpressionEngine implements ExpressionEngine
     }
 
     /**
-     * Returns a (canonic) key for the given node based on the parent's key.
+     * Returns a (canonical) key for the given node based on the parent's key.
      * This implementation will create an XPATH expression that selects the
      * given node (under the assumption that the passed in parent key is valid).
      * As the <code>nodeKey()</code> implementation of
      * <code>{@link org.apache.commons.configuration.tree.DefaultExpressionEngine DefaultExpressionEngine}</code>
      * this method will not return indices for nodes. So all child nodes of a
-     * given parent whith the same name will have the same key.
+     * given parent with the same name will have the same key.
      *
      * @param node the node for which a key is to be constructed
      * @param parentKey the key of the parent node
@@ -198,18 +246,15 @@ public class XPathExpressionEngine implements ExpressionEngine
                     "prepareAdd: key must not be null!");
         }
 
-        int index = key.length() - 1;
-        while (index >= 0 && !Character.isWhitespace(key.charAt(index)))
-        {
-            index--;
-        }
+        String addKey = key;
+        int index = findKeySeparator(addKey);
         if (index < 0)
         {
-            throw new IllegalArgumentException(
-                    "prepareAdd: Passed in key must contain a whitespace!");
+            addKey = generateKeyForAdd(root, addKey);
+            index = findKeySeparator(addKey);
         }
 
-        List nodes = query(root, key.substring(0, index).trim());
+        List nodes = query(root, addKey.substring(0, index).trim());
         if (nodes.size() != 1)
         {
             throw new IllegalArgumentException(
@@ -218,7 +263,7 @@ public class XPathExpressionEngine implements ExpressionEngine
 
         NodeAddData data = new NodeAddData();
         data.setParent((ConfigurationNode) nodes.get(0));
-        initNodeAddData(data, key.substring(index).trim());
+        initNodeAddData(data, addKey.substring(index).trim());
         return data;
     }
 
@@ -241,7 +286,7 @@ public class XPathExpressionEngine implements ExpressionEngine
     /**
      * Initializes most properties of a <code>NodeAddData</code> object. This
      * method is called by <code>prepareAdd()</code> after the parent node has
-     * been found. Its task is to interprete the passed in path of the new node.
+     * been found. Its task is to interpret the passed in path of the new node.
      *
      * @param data the data object to initialize
      * @param path the path of the new node
@@ -308,6 +353,37 @@ public class XPathExpressionEngine implements ExpressionEngine
         data.setAttribute(attr);
     }
 
+    /**
+     * Tries to generate a key for adding a property. This method is called if a
+     * key was used for adding properties which does not contain a space
+     * character. It splits the key at its single components and searches for
+     * the last existing component. Then a key compatible for adding properties
+     * is generated.
+     *
+     * @param root the root node of the configuration
+     * @param key the key in question
+     * @return the key to be used for adding the property
+     */
+    private String generateKeyForAdd(ConfigurationNode root, String key)
+    {
+        int pos = key.lastIndexOf(PATH_DELIMITER, key.length());
+
+        while (pos >= 0)
+        {
+            String keyExisting = key.substring(0, pos);
+            if (!query(root, keyExisting).isEmpty())
+            {
+                StringBuffer buf = new StringBuffer(key.length() + 1);
+                buf.append(keyExisting).append(SPACE);
+                buf.append(key.substring(pos + 1));
+                return buf.toString();
+            }
+            pos = key.lastIndexOf(PATH_DELIMITER, pos - 1);
+        }
+
+        return SPACE + key;
+    }
+
     /**
      * Helper method for throwing an exception about an invalid path.
      *
@@ -320,6 +396,23 @@ public class XPathExpressionEngine implements ExpressionEngine
                 + "\" " + msg);
     }
 
+    /**
+     * Determines the position of the separator in a key for adding new
+     * properties. If no delimiter is found, result is -1.
+     *
+     * @param key the key
+     * @return the position of the delimiter
+     */
+    private static int findKeySeparator(String key)
+    {
+        int index = key.length() - 1;
+        while (index >= 0 && !Character.isWhitespace(key.charAt(index)))
+        {
+            index--;
+        }
+        return index;
+    }
+
     // static initializer: registers the configuration node pointer factory
     static
     {
