@@ -36,7 +36,7 @@ import org.apache.commons.configuration.reloading.FileAlwaysReloadingStrategy;
 /**
  * Test loading multiple configurations.
  *
- * @version $Id: TestCompositeConfiguration.java 705028 2008-10-15 20:33:35Z oheger $
+ * @version $Id: TestCompositeConfiguration.java 1089379 2011-04-06 10:06:24Z oheger $
  */
 public class TestCompositeConfiguration extends TestCase
 {
@@ -805,6 +805,23 @@ public class TestCompositeConfiguration extends TestCase
         assertEquals("Wrong value in array", "override", values[0]);
     }
 
+    /**
+     * Tests whether interpolation works if multiple configurations are
+     * involved. This test is related to CONFIGURATION-441.
+     */
+    public void testInterpolationInMultipleConfigs()
+    {
+        Configuration c1 = new PropertiesConfiguration();
+        c1.addProperty("property.one", "one");
+        c1.addProperty("property.two", "two");
+        Configuration c2 = new PropertiesConfiguration();
+        c2.addProperty("property.one.ref", "${property.one}");
+        cc.addConfiguration(c1);
+        cc.addConfiguration(c2);
+        assertEquals("Wrong interpolated value", "one",
+                cc.getString("property.one.ref"));
+    }
+
     /**
      * A test configuration event listener that counts the number of received
      * events. Used for testing the event facilities.
