@@ -18,6 +18,7 @@
 package org.apache.commons.configuration;
 
 import java.io.File;
+import java.net.MalformedURLException;
 import java.net.URL;
 import java.util.ArrayList;
 import java.util.Iterator;
@@ -34,7 +35,7 @@ import com.mockobjects.dynamic.Mock;
 /**
  * Tests the ConfigurationUtils class
  *
- * @version $Revision: 710159 $, $Date: 2008-11-03 22:07:27 +0100 (Mo, 03 Nov 2008) $
+ * @version $Revision: 952624 $, $Date: 2010-06-08 14:00:58 +0200 (Di, 08. Jun 2010) $
  */
 public class TestConfigurationUtils extends TestCase
 {
@@ -93,16 +94,16 @@ public class TestConfigurationUtils extends TestCase
                 .toString());
         File absFile = new File("config.xml").getAbsoluteFile();
         assertEquals(
-            absFile.toURL(),
+            absFile.toURI().toURL(),
             ConfigurationUtils.getURL(
                 "http://localhost:8080/webapp/config/baseConfig.xml",
                 absFile.getAbsolutePath()));
         assertEquals(
-            absFile.toURL(),
+            absFile.toURI().toURL(),
             ConfigurationUtils.getURL(null, absFile.getAbsolutePath()));
 
-		assertEquals(absFile.toURL(),
-		ConfigurationUtils.getURL(absFile.getParent(), "config.xml"));
+        assertEquals(absFile.toURI().toURL(),
+        ConfigurationUtils.getURL(absFile.getParent(), "config.xml"));
     }
 
     public void testGetBasePath() throws Exception
@@ -187,13 +188,35 @@ public class TestConfigurationUtils extends TestCase
         assertEquals(reference, ConfigurationUtils.getFile(null, reference.getAbsolutePath()));
         assertEquals(reference, ConfigurationUtils.getFile(directory.getAbsolutePath(), reference.getAbsolutePath()));
         assertEquals(reference, ConfigurationUtils.getFile(directory.getAbsolutePath(), reference.getName()));
-        assertEquals(reference, ConfigurationUtils.getFile(directory.toURL().toString(), reference.getName()));
-        assertEquals(reference, ConfigurationUtils.getFile("invalid", reference.toURL().toString()));
+        assertEquals(reference, ConfigurationUtils.getFile(directory.toURI().toURL().toString(), reference.getName()));
+        assertEquals(reference, ConfigurationUtils.getFile("invalid", reference.toURI().toURL().toString()));
         assertEquals(reference, ConfigurationUtils.getFile(
                 "jar:file:/C:/myjar.jar!/my-config.xml/someprops.properties",
                 reference.getAbsolutePath()));
     }
 
+    /**
+     * Tests whether a "+" character in the file name is handled correctly by
+     * fileFromURL(). This test is related to CONFIGURATION-415.
+     */
+    public void testFileFromURLWithPlus() throws MalformedURLException
+    {
+        File file = new File(new File("target"), "foo+bar.txt")
+                .getAbsoluteFile();
+        URL fileURL = file.toURI().toURL();
+        File file2 = ConfigurationUtils.fileFromURL(fileURL);
+        assertEquals("Wrong file", file, file2);
+    }
+
+    /**
+     * Tests whether fileFromURL() handles null URLs correctly.
+     */
+    public void testFileFromURLNull() throws MalformedURLException
+    {
+        assertNull("Wrong file for null URL", ConfigurationUtils
+                .fileFromURL(null));
+    }
+
     public void testLocateWithNullTCCL() throws Exception
     {
         ClassLoader cl = Thread.currentThread().getContextClassLoader();
