@@ -24,7 +24,9 @@ import java.io.FileWriter;
 import java.io.IOException;
 import java.io.PrintWriter;
 import java.net.URL;
+import java.util.Collection;
 import java.util.Iterator;
+import java.util.LinkedList;
 import java.util.Properties;
 
 import junit.framework.TestCase;
@@ -34,7 +36,7 @@ import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
 
 /**
  * @author Emmanuel Bourg
- * @version $Revision: 711782 $, $Date: 2008-11-06 08:23:41 +0100 (Do, 06 Nov 2008) $
+ * @version $Id: TestFileConfiguration.java 1029163 2010-10-30 19:04:02Z oheger $
  */
 public class TestFileConfiguration extends TestCase
 {
@@ -50,9 +52,81 @@ public class TestFileConfiguration extends TestCase
     /** Constant for a test file.*/
     private static final File TEST_FILE = new File(TEST_DIR, TEST_FILENAME);
 
+    /** Constant for a test output file. */
+    private static final File OUT_FILE = new File(
+            "target/test-resources/foo/bar/test.properties");
+
     /** Constant for the name of a resource to be resolved.*/
     private static final String RESOURCE_NAME = "config/deep/deeptest.properties";
 
+    /** A list with temporary files created during a test case. */
+    private Collection tempFiles = new LinkedList();
+
+    /**
+     * Initializes the test environment. This implementation ensures that the
+     * test output file does not exist.
+     */
+    protected void setUp() throws Exception
+    {
+        super.setUp();
+        removeOutFile();
+    }
+
+    /**
+     * Performs cleanup after a test case. This implementation removes temporary
+     * files that have been created.
+     */
+    protected void tearDown() throws Exception
+    {
+        for(Iterator it = tempFiles.iterator(); it.hasNext();)
+        {
+            File file = (File) it.next();
+            removeFile(file);
+        }
+        removeOutFile();
+
+        super.tearDown();
+    }
+
+    /**
+     * Adds a temporary file used by a test case. This method removes the file
+     * if it already exists. It is then added to a list so that it is removed at
+     * the end of the test.
+     *
+     * @param file the temporary file
+     */
+    private void addTemporaryFile(File file)
+    {
+        removeFile(file);
+        tempFiles.add(file);
+    }
+
+    /**
+     * Removes a file if it exists.
+     *
+     * @param file the file to be removed
+     */
+    private static void removeFile(File file)
+    {
+        if (file.exists())
+        {
+            assertTrue("Cannot remove file: " + file, file.delete());
+        }
+    }
+
+    /**
+     * Removes the test output file if it exists. Its parent directories are
+     * also removed.
+     */
+    private static void removeOutFile()
+    {
+        removeFile(OUT_FILE);
+        File parent = OUT_FILE.getParentFile();
+        removeFile(parent);
+        parent = parent.getParentFile();
+        removeFile(parent);
+    }
+
     public void testSetURL() throws Exception
     {
         // http URL
@@ -62,9 +136,9 @@ public class TestFileConfiguration extends TestCase
         assertEquals("base path", "http://commons.apache.org/configuration/", config.getBasePath());
         assertEquals("file name", "index.html", config.getFileName());
 
-        // file URL
+        // file URL - This url is invalid, a valid url would be file:///temp/test.properties.
         config.setURL(new URL("file:/temp/test.properties"));
-        assertEquals("base path", "file:/temp/", config.getBasePath());
+        assertEquals("base path", "file:///temp/", config.getBasePath());
         assertEquals("file name", "test.properties", config.getFileName());
     }
 
@@ -93,7 +167,7 @@ public class TestFileConfiguration extends TestCase
         assertEquals(TEST_FILENAME, config.getFileName());
         assertEquals(directory.getAbsolutePath(), config.getBasePath());
         assertEquals(file.getAbsolutePath(), config.getPath());
-        assertEquals(file.toURL(), config.getURL());
+        assertEquals(file.toURI().toURL(), config.getURL());
 
         config.setBasePath(null);
         config.setFileName(TEST_FILENAME);
@@ -103,54 +177,29 @@ public class TestFileConfiguration extends TestCase
 
     public void testCreateFile1() throws Exception
     {
-        File file = new File("target/test-resources/foo/bar/test.properties");
-        if (file.exists())
-        {
-            file.delete();
-            file.getParentFile().delete();
-        }
+        assertFalse("The file should not exist", OUT_FILE.exists());
 
-        assertFalse("The file should not exist", file.exists());
-
-        FileConfiguration config = new PropertiesConfiguration(file);
+        FileConfiguration config = new PropertiesConfiguration(OUT_FILE);
         config.save();
 
-        assertTrue("The file doesn't exist", file.exists());
+        assertTrue("The file doesn't exist", OUT_FILE.exists());
     }
 
     public void testCreateFile2() throws Exception
     {
-        File file = new File("target/test-resources/foo/bar/test.properties");
-        if (file.exists())
-        {
-            file.delete();
-            file.getParentFile().delete();
-        }
-
-        assertFalse("The file should not exist", file.exists());
-
         FileConfiguration config = new PropertiesConfiguration();
-        config.setFile(file);
+        config.setFile(OUT_FILE);
         config.save();
 
-        assertTrue("The file doesn't exist", file.exists());
+        assertTrue("The file doesn't exist", OUT_FILE.exists());
     }
 
     public void testCreateFile3() throws Exception
     {
-        File file = new File("target/test-resources/foo/bar/test.properties");
-        if (file.exists())
-        {
-            file.delete();
-            file.getParentFile().delete();
-        }
-
-        assertFalse("The file should not exist", file.exists());
-
         FileConfiguration config = new PropertiesConfiguration();
-        config.save(file);
+        config.save(OUT_FILE);
 
-        assertTrue("The file doesn't exist", file.exists());
+        assertTrue("The file doesn't exist", OUT_FILE.exists());
     }
 
     /**
@@ -163,43 +212,31 @@ public class TestFileConfiguration extends TestCase
     {
         File dir = new File("conf");
         File file = new File(dir, "testFileConfiguration.properties");
+        addTemporaryFile(file);
 
-        if (file.exists())
+        ConfigurationFactory factory = new ConfigurationFactory();
+        factory.setConfigurationURL(new File(dir,
+                "testDigesterConfiguration2.xml").toURI().toURL());
+        CompositeConfiguration cc =
+                (CompositeConfiguration) factory.getConfiguration();
+        PropertiesConfiguration config = null;
+        for (int i = 0; config == null; i++)
         {
-            assertTrue("File cannot be deleted", file.delete());
-        }
-
-        try
-        {
-            ConfigurationFactory factory = new ConfigurationFactory();
-            factory.setConfigurationURL(new File(dir, "testDigesterConfiguration2.xml").toURL());
-            CompositeConfiguration cc = (CompositeConfiguration) factory.getConfiguration();
-            PropertiesConfiguration config = null;
-            for (int i = 0; config == null; i++)
+            if (cc.getConfiguration(i) instanceof PropertiesConfiguration)
             {
-                if (cc.getConfiguration(i) instanceof PropertiesConfiguration)
-                {
-                    config = (PropertiesConfiguration) cc.getConfiguration(i);
-                }
-            }
-
-            config.setProperty("test", "yes");
-            config.save(file.getName());
-            assertTrue(file.exists());
-            config = new PropertiesConfiguration();
-            config.setFile(file);
-            config.load();
-
-            assertEquals("yes", config.getProperty("test"));
-            assertEquals("masterOfPost", config.getProperty("mail.account.user"));
-        }
-        finally
-        {
-            if (file.exists())
-            {
-                assertTrue("File could not be deleted", file.delete());
+                config = (PropertiesConfiguration) cc.getConfiguration(i);
             }
         }
+
+        config.setProperty("test", "yes");
+        config.save(file.getName());
+        assertTrue(file.exists());
+        config = new PropertiesConfiguration();
+        config.setFile(file);
+        config.load();
+
+        assertEquals("yes", config.getProperty("test"));
+        assertEquals("masterOfPost", config.getProperty("mail.account.user"));
     }
 
     /**
@@ -243,6 +280,7 @@ public class TestFileConfiguration extends TestCase
             String path = System.getProperties().getProperty("user.home");
             File homeDir = new File(path);
             tempFile = File.createTempFile("CONF", null, homeDir);
+            tempFiles.add(tempFile);
             String fileName = tempFile.getName();
             Properties props = new Properties();
             props.setProperty("1", "one");
@@ -286,10 +324,6 @@ public class TestFileConfiguration extends TestCase
                     ioex.printStackTrace();
                 }
             }
-            if (tempFile.exists())
-            {
-                assertTrue(tempFile.delete());
-            }
         }
     }
 
@@ -300,6 +334,7 @@ public class TestFileConfiguration extends TestCase
     public void testReloadingWithAutoSave() throws Exception
     {
         File configFile = new File(TARGET_DIR, TEST_FILENAME);
+        addTemporaryFile(configFile);
         PrintWriter out = null;
 
         try
@@ -324,10 +359,6 @@ public class TestFileConfiguration extends TestCase
             {
                 out.close();
             }
-            if (configFile.exists())
-            {
-                assertTrue(configFile.delete());
-            }
         }
     }
 
@@ -339,20 +370,19 @@ public class TestFileConfiguration extends TestCase
     {
         File path = new File(TARGET_DIR, "path with spaces");
         File confFile = new File(path, "config-test.properties");
+        addTemporaryFile(confFile);
+        addTemporaryFile(path);
         PrintWriter out = null;
 
         try
         {
-            if (!path.exists())
-            {
-                assertTrue(path.mkdir());
-            }
+            assertTrue(path.mkdir());
             out = new PrintWriter(new FileWriter(confFile));
             out.println("saved = false");
             out.close();
             out = null;
 
-            URL url = new URL(TARGET_DIR.toURL()
+            URL url = new URL(TARGET_DIR.toURI().toURL()
                     + "path%20with%20spaces/config-test.properties");
             PropertiesConfiguration config = new PropertiesConfiguration(url);
             config.load();
@@ -371,17 +401,27 @@ public class TestFileConfiguration extends TestCase
             {
                 out.close();
             }
-            if (confFile.exists())
-            {
-                assertTrue(confFile.delete());
-            }
-            if (path.exists())
-            {
-                assertTrue(path.delete());
-            }
         }
     }
 
+    /**
+     * Tests whether file names containing a "+" character are handled
+     * correctly. This test is related to CONFIGURATION-415.
+     */
+    public void testPathWithPlus() throws ConfigurationException, IOException
+    {
+        File saveFile =
+                new File(TARGET_DIR, "test+config.properties")
+                        .getAbsoluteFile();
+        saveFile.createNewFile();
+        tempFiles.add(saveFile);
+        FileConfiguration config = new PropertiesConfiguration(saveFile);
+        config.addProperty("test", Boolean.TRUE);
+        config.save();
+        File configFile = config.getFile();
+        assertEquals("Wrong configuration file", saveFile, configFile);
+    }
+
     /**
      * Tests the getFile() method.
      */
@@ -458,7 +498,7 @@ public class TestFileConfiguration extends TestCase
         }
 
         config = new PropertiesConfiguration();
-        config.load(file.toURL());
+        config.load(file.toURI().toURL());
         try
         {
             config.save();
@@ -612,6 +652,38 @@ public class TestFileConfiguration extends TestCase
         checkIterationWithReload(config);
     }
 
+    /**
+     * Tests whether a configuration can be refreshed.
+     */
+    public void testRefresh() throws ConfigurationException
+    {
+        PropertiesConfiguration config = new PropertiesConfiguration(TEST_FILE);
+        assertEquals("Wrong value", 10, config.getInt("test.integer"));
+        config.setProperty("test.integer", new Integer(42));
+        assertEquals("Wrong value after update", 42,
+                config.getInt("test.integer"));
+        config.refresh();
+        assertEquals("Wrong value after refresh", 10,
+                config.getInt("test.integer"));
+    }
+
+    /**
+     * Tests refresh if the configuration is not associated with a file.
+     */
+    public void testRefreshNoFile() throws ConfigurationException
+    {
+        PropertiesConfiguration config = new PropertiesConfiguration();
+        try
+        {
+            config.refresh();
+            fail("Could refresh configuration without a file!");
+        }
+        catch (ConfigurationException cex)
+        {
+            // ok
+        }
+    }
+
     /**
      * Helper method for testing an iteration over the keys of a file-based
      * configuration while a reload happens.
