@@ -38,7 +38,7 @@ import org.apache.commons.configuration.tree.ExpressionEngine;
 /**
  * Test class for HierarchicalConfiguration.
  *
- * @version $Id: TestHierarchicalConfiguration.java 722238 2008-12-01 21:28:31Z oheger $
+ * @version $Id: TestHierarchicalConfiguration.java 1152923 2011-08-01 20:28:03Z oheger $
  */
 public class TestHierarchicalConfiguration extends TestCase
 {
@@ -189,6 +189,14 @@ public class TestHierarchicalConfiguration extends TestCase
         assertEquals(42, config.getInt("test.items.item"));
     }
 
+    public void testClear()
+    {
+        config.setProperty(null, "value");
+        config.addProperty("[@attr]", "defined");
+        config.clear();
+        assertTrue("Configuration not empty", config.isEmpty());
+    }
+
     public void testClearProperty()
     {
         config.clearProperty("tables.table(0).fields.field(0).name");
@@ -579,6 +587,24 @@ public class TestHierarchicalConfiguration extends TestCase
         }
     }
 
+    /**
+     * Tests whether a sub configuration obtained by configurationAt() can be
+     * cleared.
+     */
+    public void testConfigurationAtClear()
+    {
+        config.addProperty("test.sub.test", "fail");
+        assertEquals("Wrong index (1)", 0, config.getMaxIndex("test"));
+        SubnodeConfiguration sub = config.configurationAt("test.sub");
+        assertEquals("Wrong value", "fail", sub.getString("test"));
+        sub.clear();
+        assertNull("Key still found", config.getString("test.sub.key"));
+        sub.setProperty("test", "success");
+        assertEquals("Property not set", "success",
+                config.getString("test.sub.test"));
+        assertEquals("Wrong index (2)", 0, config.getMaxIndex("test"));
+    }
+
     /**
      * Tests the configurationsAt() method.
      */
@@ -766,8 +792,27 @@ public class TestHierarchicalConfiguration extends TestCase
     {
         CountVisitor v = new CountVisitor();
         config.getRoot().visit(v, null);
-        assertEquals(28, v.beforeCount);
-        assertEquals(v.beforeCount, v.afterCount);
+        assertEquals("Wrong number of visits", 28, v.beforeCount);
+        assertEquals("Different number of before and after visits",
+                v.beforeCount, v.afterCount);
+    }
+
+    /**
+     * Tests the visitor mechanism if a ConfigurationKey is passed in.
+     */
+    public void testNodeVisitorKeys()
+    {
+        CountVisitor v = new CountVisitor();
+        ConfigurationKey configKey = new ConfigurationKey();
+        config.getRoot().visit(v, configKey);
+        for (Iterator it = config.getKeys(); it.hasNext();)
+        {
+            String key = (String) it.next();
+            assertTrue("Key not found in before keys: " + key, v.beforeKeys
+                    .contains(key));
+            assertTrue("Key not found in after keys: " + key, v.afterKeys
+                    .contains(key));
+        }
     }
 
     /**
@@ -815,25 +860,25 @@ public class TestHierarchicalConfiguration extends TestCase
         }
     }
 
-	/**
+    /**
      * Tests the copy constructor.
      */
-	public void testInitCopy()
-	{
-		HierarchicalConfiguration copy = new HierarchicalConfiguration(config);
-		checkContent(copy);
-	}
+    public void testInitCopy()
+    {
+        HierarchicalConfiguration copy = new HierarchicalConfiguration(config);
+        checkContent(copy);
+    }
 
-	/**
+    /**
      * Tests whether the nodes of a copied configuration are independent from
      * the source configuration.
      */
-	public void testInitCopyUpdate()
-	{
-		HierarchicalConfiguration copy = new HierarchicalConfiguration(config);
-		config.setProperty("tables.table(0).name", "NewTable");
-		checkContent(copy);
-	}
+    public void testInitCopyUpdate()
+    {
+        HierarchicalConfiguration copy = new HierarchicalConfiguration(config);
+        config.setProperty("tables.table(0).name", "NewTable");
+        checkContent(copy);
+    }
 
     /**
      * Tests interpolation facilities.
@@ -887,6 +932,20 @@ public class TestHierarchicalConfiguration extends TestCase
         InterpolationTestHelper.testInterpolationSubset(config);
     }
 
+    /**
+     * Tests whether interpolation with a subset configuration works over
+     * multiple layers.
+     */
+    public void testInterpolationSubsetMultipleLayers()
+    {
+        config.clear();
+        config.addProperty("var", "value");
+        config.addProperty("prop2.prop[@attr]", "${var}");
+        Configuration sub1 = config.subset("prop2");
+        Configuration sub2 = sub1.subset("prop");
+        assertEquals("Wrong value", "value", sub2.getString("[@attr]"));
+    }
+
     /**
      * Tests interpolation of a variable, which cannot be resolved.
      */
@@ -940,16 +999,16 @@ public class TestHierarchicalConfiguration extends TestCase
         testGetProperty();
     }
 
-	/**
+    /**
      * Tests the copy constructor when a null reference is passed.
      */
-	public void testInitCopyNull()
-	{
-		HierarchicalConfiguration copy = new HierarchicalConfiguration(null);
-		assertTrue("Configuration not empty", copy.isEmpty());
-	}
+    public void testInitCopyNull()
+    {
+        HierarchicalConfiguration copy = new HierarchicalConfiguration(null);
+        assertTrue("Configuration not empty", copy.isEmpty());
+    }
 
-	/**
+    /**
      * Tests the parents of nodes when setRootNode() is involved. This is
      * related to CONFIGURATION-334.
      */
@@ -986,7 +1045,17 @@ public class TestHierarchicalConfiguration extends TestCase
         assertEquals("Wrong number of children", 2, oldRoot.getChildrenCount());
     }
 
-	/**
+    /**
+     * Tests whether keys that contains brackets can be used.
+     */
+    public void testGetPropertyKeyWithBrackets()
+    {
+        final String key = "test.directory.platform(x86)";
+        config.addProperty(key, "C:\\Temp");
+        assertEquals("Wrong property value", "C:\\Temp", config.getString(key));
+    }
+
+    /**
      * Helper method for testing the getKeys(String) method.
      *
      * @param prefix the key to pass into getKeys()
@@ -1052,27 +1121,27 @@ public class TestHierarchicalConfiguration extends TestCase
                 .contains("tables/table/fields/field/name"));
     }
 
-	/**
+    /**
      * Checks the content of the passed in configuration object. Used by some
      * tests that copy a configuration.
      *
      * @param c the configuration to check
      */
-	private void checkContent(Configuration c)
-	{
-		for (int i = 0; i < tables.length; i++)
-		{
-			assertEquals(tables[i], c.getString("tables.table(" + i + ").name"));
-			for (int j = 0; j < fields[i].length; j++)
-			{
-				assertEquals(fields[i][j], c.getString("tables.table(" + i
-						+ ").fields.field(" + j + ").name"));
-			}
-		}
-	}
+    private void checkContent(Configuration c)
+    {
+        for (int i = 0; i < tables.length; i++)
+        {
+            assertEquals(tables[i], c.getString("tables.table(" + i + ").name"));
+            for (int j = 0; j < fields[i].length; j++)
+            {
+                assertEquals(fields[i][j], c.getString("tables.table(" + i
+                        + ").fields.field(" + j + ").name"));
+            }
+        }
+    }
 
-	private ExpressionEngine createAlternativeExpressionEngine()
-	{
+    private ExpressionEngine createAlternativeExpressionEngine()
+    {
         DefaultExpressionEngine engine = new DefaultExpressionEngine();
         engine.setPropertyDelimiter("/");
         engine.setIndexStart("[");
@@ -1112,20 +1181,36 @@ public class TestHierarchicalConfiguration extends TestCase
      */
     static class CountVisitor extends HierarchicalConfiguration.NodeVisitor
     {
-        public int beforeCount;
+        /** The number of invocations of visitBeforeChildren(). */
+        int beforeCount;
 
-        public int afterCount;
+        /** The number of invocations of visitAfterChildren(). */
+        int afterCount;
+
+        /** A set with the keys passed to visitBeforeChildren(). */
+        final Set beforeKeys = new HashSet();
+
+        /** A set with the keys passed to visitAfterChildren(). */
+        final Set afterKeys = new HashSet();
 
         public void visitAfterChildren(Node node, ConfigurationKey key)
         {
             super.visitAfterChildren(node, key);
             afterCount++;
+            if (key != null)
+            {
+                afterKeys.add(key.toString());
+            }
         }
 
         public void visitBeforeChildren(Node node, ConfigurationKey key)
         {
             super.visitBeforeChildren(node, key);
             beforeCount++;
+            if (key != null)
+            {
+                beforeKeys.add(key.toString());
+            }
         }
     }
 }
