@@ -18,9 +18,11 @@
 package org.apache.commons.configuration;
 
 import java.util.Hashtable;
+import java.util.Properties;
 
 import javax.naming.Context;
 import javax.naming.InitialContext;
+import javax.naming.NameNotFoundException;
 import javax.naming.NamingException;
 
 import junit.framework.TestCase;
@@ -30,13 +32,13 @@ import org.apache.commons.configuration.event.ConfigurationErrorListener;
 /**
  * Test to see if the JNDIConfiguration works properly.
  *
- * @version $Id: TestJNDIConfiguration.java 558330 2007-07-21 15:26:19Z oheger $
+ * @version $Id: TestJNDIConfiguration.java 1081926 2011-03-15 20:20:25Z oheger $
  */
 public class TestJNDIConfiguration extends TestCase {
 
     public static final String CONTEXT_FACTORY = MockInitialContextFactory.class.getName();
 
-    private JNDIConfiguration conf;
+    private PotentialErrorJNDIConfiguration conf;
     private NonStringTestHolder nonStringTestHolder;
 
     /** A test error listener for counting internal errors.*/
@@ -46,7 +48,10 @@ public class TestJNDIConfiguration extends TestCase {
 
         System.setProperty("java.naming.factory.initial", CONTEXT_FACTORY);
 
-        conf = new PotentialErrorJNDIConfiguration();
+        Properties props = new Properties();
+        props.put("java.naming.factory.initial", CONTEXT_FACTORY);
+        Context ctx = new InitialContext(props);
+        conf = new PotentialErrorJNDIConfiguration(ctx);
 
         nonStringTestHolder = new NonStringTestHolder();
         nonStringTestHolder.setConfiguration(conf);
@@ -172,14 +177,14 @@ public class TestJNDIConfiguration extends TestCase {
     public void testConstructor() throws Exception
     {
         // test the constructor accepting a context
-        conf = new JNDIConfiguration(new InitialContext());
+        JNDIConfiguration c = new JNDIConfiguration(new InitialContext());
 
-        assertEquals("'test.boolean' property", "true", conf.getString("test.boolean"));
+        assertEquals("'test.boolean' property", "true", c.getString("test.boolean"));
 
         // test the constructor accepting a context and a prefix
-        conf = new JNDIConfiguration(new InitialContext(), "test");
+        c = new JNDIConfiguration(new InitialContext(), "test");
 
-        assertEquals("'boolean' property", "true", conf.getString("boolean"));
+        assertEquals("'boolean' property", "true", c.getString("boolean"));
     }
 
     /**
@@ -187,7 +192,7 @@ public class TestJNDIConfiguration extends TestCase {
      */
     private PotentialErrorJNDIConfiguration setUpErrorConfig()
     {
-        ((PotentialErrorJNDIConfiguration) conf).failOnGetCtx = true;
+        conf.installException();
         conf.removeErrorListener((ConfigurationErrorListener) conf
                 .getErrorListeners().iterator().next());
         return (PotentialErrorJNDIConfiguration) conf;
@@ -213,8 +218,8 @@ public class TestJNDIConfiguration extends TestCase {
      */
     public void testLogListener() throws NamingException
     {
-        conf = new JNDIConfiguration();
-        assertEquals("No error log listener registered", 1, conf
+        JNDIConfiguration c = new JNDIConfiguration();
+        assertEquals("No error log listener registered", 1, c
                 .getErrorListeners().size());
     }
 
@@ -269,8 +274,19 @@ public class TestJNDIConfiguration extends TestCase {
         Hashtable env = new Hashtable();
         env.put(MockInitialContextFactory.PROP_CYCLES, Boolean.TRUE);
         InitialContext initCtx = new InitialContext(env);
-        conf = new JNDIConfiguration(initCtx);
-        conf.getKeys("cycle");
+        JNDIConfiguration c = new JNDIConfiguration(initCtx);
+        c.getKeys("cycle");
+    }
+
+    /**
+     * Tests getKeys() if no data is found. This should not cause a problem and
+     * not notify the error listeners.
+     */
+    public void testGetKeysNoData()
+    {
+        conf.installException(new NameNotFoundException("Test exception"));
+        assertFalse("Got keys", conf.getKeys().hasNext());
+        listener.verify();
     }
 
     /**
@@ -278,26 +294,46 @@ public class TestJNDIConfiguration extends TestCase {
      * throw an exception when accessing the base context. Used for testing the
      * exception handling.
      */
-    static class PotentialErrorJNDIConfiguration extends JNDIConfiguration
+    public static class PotentialErrorJNDIConfiguration extends
+            JNDIConfiguration
     {
-        /** A flag whether an exception should be thrown. */
-        boolean failOnGetCtx;
+        /** An exception to be thrown by getBaseContext(). */
+        private NamingException exception;
 
-        public PotentialErrorJNDIConfiguration() throws NamingException
-        {
-            super();
-        }
-
-        public PotentialErrorJNDIConfiguration(Context ctx) throws NamingException
+        public PotentialErrorJNDIConfiguration(Context ctx)
+                throws NamingException
         {
             super(ctx);
         }
 
+        /**
+         * Prepares this object to throw an exception when the JNDI context is
+         * queried.
+         *
+         * @param nex the exception to be thrown
+         */
+        public void installException(NamingException nex)
+        {
+            exception = nex;
+        }
+
+        /**
+         * Prepares this object to throw a standard exception when the JNDI
+         * context is queried.
+         */
+        public void installException()
+        {
+            installException(new NamingException("Simulated JNDI exception!"));
+        }
+
+        /**
+         * Returns the JNDI context. Optionally throws an exception.
+         */
         public Context getBaseContext() throws NamingException
         {
-            if (failOnGetCtx)
+            if (exception != null)
             {
-                throw new NamingException("Simulated JNDI exception!");
+                throw exception;
             }
             return super.getBaseContext();
         }
