@@ -17,8 +17,10 @@
 
 package org.apache.commons.configuration;
 
+import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileOutputStream;
+import java.io.FileReader;
 import java.io.FileWriter;
 import java.io.IOException;
 import java.io.InputStream;
@@ -27,13 +29,16 @@ import java.io.PrintWriter;
 import java.io.Reader;
 import java.io.StringReader;
 import java.io.StringWriter;
+import java.io.Writer;
 import java.net.HttpURLConnection;
 import java.net.URL;
 import java.net.URLConnection;
 import java.net.URLStreamHandler;
 import java.util.ArrayList;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Set;
 
 import junit.framework.TestCase;
 
@@ -43,18 +48,25 @@ import org.apache.commons.lang.SystemUtils;
 /**
  * Test for loading and saving properties files.
  *
- * @version $Id: TestPropertiesConfiguration.java 712431 2008-11-08 21:05:23Z oheger $
+ * @version $Id: TestPropertiesConfiguration.java 951878 2010-06-06 15:56:47Z oheger $
  */
 public class TestPropertiesConfiguration extends TestCase
 {
+    /** Constant for a test property name.*/
+    private static final String PROP_NAME = "testProperty";
+
+    /** Constant for a test property value.*/
+    private static final String PROP_VALUE = "value";
+
+    /** The configuration to be tested.*/
     private PropertiesConfiguration conf;
 
     /** The File that we test with */
-    private String testProperties = new File("conf/test.properties").getAbsolutePath();
+    private static String testProperties = new File("conf/test.properties").getAbsolutePath();
 
-    private String testBasePath = new File("conf").getAbsolutePath();
-    private String testBasePath2 = new File("conf").getAbsoluteFile().getParentFile().getAbsolutePath();
-    private File testSavePropertiesFile = new File("target/testsave.properties");
+    private static String testBasePath = new File("conf").getAbsolutePath();
+    private static String testBasePath2 = new File("conf").getAbsoluteFile().getParentFile().getAbsolutePath();
+    private static File testSavePropertiesFile = new File("target/testsave.properties");
 
     protected void setUp() throws Exception
     {
@@ -250,7 +262,8 @@ public class TestPropertiesConfiguration extends TestCase
     public void testSaveWithBasePath() throws Exception
     {
         conf.setProperty("test", "true");
-        conf.setBasePath(testSavePropertiesFile.getParentFile().toURL().toString());
+        conf.setBasePath(testSavePropertiesFile.getParentFile().toURI().toURL()
+                .toString());
         conf.setFileName(testSavePropertiesFile.getName());
         conf.save();
         assertTrue(testSavePropertiesFile.exists());
@@ -428,12 +441,7 @@ public class TestPropertiesConfiguration extends TestCase
             throws ConfigurationException
     {
         PropertiesConfiguration checkConfig = new PropertiesConfiguration(testSavePropertiesFile);
-        for (Iterator i = conf.getKeys(); i.hasNext();)
-        {
-            String key = (String) i.next();
-            assertTrue("The saved configuration doesn't contain the key '" + key + "'", checkConfig.containsKey(key));
-            assertEquals("Value of the '" + key + "' property", conf.getProperty(key), checkConfig.getProperty(key));
-        }
+        ConfigurationAssert.assertEquals(conf, checkConfig);
         return checkConfig;
     }
 
@@ -856,6 +864,163 @@ public class TestPropertiesConfiguration extends TestCase
                 testConf.loadEncoding);
     }
 
+    /**
+     * Tests whether a default IOFactory is set.
+     */
+    public void testGetIOFactoryDefault()
+    {
+        assertNotNull("No default IO factory", conf.getIOFactory());
+    }
+
+    /**
+     * Tests setting the IOFactory to null. This should cause an exception.
+     */
+    public void testSetIOFactoryNull()
+    {
+        try
+        {
+            conf.setIOFactory(null);
+            fail("Could set IO factory to null!");
+        }
+        catch (IllegalArgumentException iex)
+        {
+            // ok
+        }
+    }
+
+    /**
+     * Tests setting an IOFactory that uses a specialized reader.
+     */
+    public void testSetIOFactoryReader() throws ConfigurationException
+    {
+        final int propertyCount = 10;
+        conf.clear();
+        conf.setIOFactory(new PropertiesConfiguration.IOFactory()
+        {
+            public PropertiesConfiguration.PropertiesReader createPropertiesReader(
+                    Reader in, char delimiter)
+            {
+                return new PropertiesReaderTestImpl(in, delimiter,
+                        propertyCount);
+            }
+
+            public PropertiesConfiguration.PropertiesWriter createPropertiesWriter(
+                    Writer out, char delimiter)
+            {
+                throw new UnsupportedOperationException("Unexpected call!");
+            }
+        });
+        conf.load();
+        for (int i = 1; i <= propertyCount; i++)
+        {
+            assertEquals("Wrong property value at " + i, PROP_VALUE + i, conf
+                    .getString(PROP_NAME + i));
+        }
+    }
+
+    /**
+     * Tests setting an IOFactory that uses a specialized writer.
+     */
+    public void testSetIOFactoryWriter() throws ConfigurationException, IOException
+    {
+        final PropertiesWriterTestImpl testWriter = new PropertiesWriterTestImpl(',');
+        conf.setIOFactory(new PropertiesConfiguration.IOFactory()
+        {
+            public PropertiesConfiguration.PropertiesReader createPropertiesReader(
+                    Reader in, char delimiter)
+            {
+                throw new UnsupportedOperationException("Unexpected call!");
+            }
+
+            public PropertiesConfiguration.PropertiesWriter createPropertiesWriter(
+                    Writer out, char delimiter)
+            {
+                return testWriter;
+            }
+        });
+        conf.save(new StringWriter());
+        testWriter.close();
+        checkSavedConfig();
+    }
+
+    /**
+     * Tests that the property separators are retained when saving the
+     * configuration.
+     */
+    public void testKeepSeparators() throws ConfigurationException, IOException
+    {
+        conf.save(testSavePropertiesFile);
+        final String[] separatorTests = {
+                "test.separator.equal = foo", "test.separator.colon : foo",
+                "test.separator.tab\tfoo", "test.separator.whitespace foo",
+                "test.separator.no.space=foo"
+        };
+        Set foundLines = new HashSet();
+        BufferedReader in = new BufferedReader(new FileReader(
+                testSavePropertiesFile));
+        try
+        {
+            String s;
+            while ((s = in.readLine()) != null)
+            {
+                for (int i = 0; i < separatorTests.length; i++)
+                {
+                    if (separatorTests[i].equals(s))
+                    {
+                        foundLines.add(s);
+                    }
+                }
+            }
+        }
+        finally
+        {
+            in.close();
+        }
+        assertEquals("No all separators were found: " + foundLines,
+                separatorTests.length, foundLines.size());
+    }
+
+    /**
+     * Tests whether properties with slashes in their values can be saved. This
+     * test is related to CONFIGURATION-408.
+     */
+    public void testSlashEscaping() throws ConfigurationException
+    {
+        conf.setProperty(PROP_NAME, "http://www.apache.org");
+        StringWriter writer = new StringWriter();
+        conf.save(writer);
+        String s = writer.toString();
+        assertTrue("Value not found: " + s, s.indexOf(PROP_NAME
+                + " = http://www.apache.org") >= 0);
+    }
+
+    /**
+     * Tests whether backslashes are correctly handled if lists are parsed. This
+     * test is related to CONFIGURATION-418.
+     */
+    public void testBackslashEscapingInLists() throws ConfigurationException
+    {
+        checkBackslashList("share2");
+        checkBackslashList("share1");
+    }
+
+    /**
+     * Helper method for testing the content of a list with elements that
+     * contain backslashes.
+     *
+     * @param key the key
+     */
+    private void checkBackslashList(String key)
+    {
+        Object prop = conf.getProperty("test." + key);
+        assertTrue("Not a list", prop instanceof List);
+        List list = (List) prop;
+        assertEquals("Wrong number of list elements", 2, list.size());
+        final String prefix = "\\\\" + key;
+        assertEquals("Wrong element 1", prefix + "a", list.get(0));
+        assertEquals("Wrong element 2", prefix + "b", list.get(1));
+    }
+
     /**
      * Creates a configuration that can be used for testing copy operations.
      *
@@ -987,4 +1152,57 @@ public class TestPropertiesConfiguration extends TestCase
             return connection;
         }
     }
+
+    /**
+     * A test PropertiesReader for testing whether a custom reader can be
+     * injected. This implementation creates a configurable number of synthetic
+     * test properties.
+     */
+    private static class PropertiesReaderTestImpl extends
+            PropertiesConfiguration.PropertiesReader
+    {
+        /** The number of test properties to be created. */
+        private final int maxProperties;
+
+        /** The current number of properties. */
+        private int propertyCount;
+
+        public PropertiesReaderTestImpl(Reader reader, char listDelimiter,
+                int maxProps)
+        {
+            super(reader, listDelimiter);
+            assertEquals("Wrong list delimiter", ',', listDelimiter);
+            maxProperties = maxProps;
+        }
+
+        public String getPropertyName()
+        {
+            return PROP_NAME + propertyCount;
+        }
+
+        public String getPropertyValue()
+        {
+            return PROP_VALUE + propertyCount;
+        }
+
+        public boolean nextProperty() throws IOException
+        {
+            propertyCount++;
+            return propertyCount <= maxProperties;
+        }
+    }
+
+    /**
+     * A test PropertiesWriter for testing whether a custom writer can be
+     * injected. This implementation simply redirects all output into a test
+     * file.
+     */
+    private static class PropertiesWriterTestImpl extends
+            PropertiesConfiguration.PropertiesWriter
+    {
+        public PropertiesWriterTestImpl(char delimiter) throws IOException
+        {
+            super(new FileWriter(testSavePropertiesFile), delimiter);
+        }
+    }
 }
