@@ -21,22 +21,25 @@ import java.io.StringReader;
 import java.io.StringWriter;
 import java.util.Iterator;
 
-import org.apache.commons.configuration.event.ConfigurationEvent;
-
 import junit.framework.TestCase;
 
+import org.apache.commons.configuration.event.ConfigurationEvent;
+
 /**
  * Test class for PropertiesConfigurationLayout.
  *
  * @author <a
  * href="http://commons.apache.org/configuration/team-list.html">Commons
  * Configuration team</a>
- * @version $Id: TestPropertiesConfigurationLayout.java 561230 2007-07-31 04:17:09Z rahul $
+ * @version $Id: TestPropertiesConfigurationLayout.java 759750 2009-03-29 19:15:36Z oheger $
  */
 public class TestPropertiesConfigurationLayout extends TestCase
 {
     /** Constant for the line break character. */
-    static final String CR = System.getProperty("line.separator");
+    private static final String CR = System.getProperty("line.separator");
+
+    /** Constant for the normalized line break character. */
+    private static final String CRNORM = "\n";
 
     /** Constant for a test property key. */
     static final String TEST_KEY = "myProperty";
@@ -79,6 +82,7 @@ public class TestPropertiesConfigurationLayout extends TestCase
         assertSame("Configuration not stored", config, layout
                 .getConfiguration());
         assertFalse("Force single line flag set", layout.isForceSingleLine());
+        assertNull("Got a global separator", layout.getGlobalSeparator());
     }
 
     /**
@@ -132,7 +136,7 @@ public class TestPropertiesConfigurationLayout extends TestCase
         layout.load(builder.getReader());
         assertEquals("Wrong number of blanc lines", 2, layout
                 .getBlancLinesBefore(TEST_KEY));
-        assertEquals("Wrong comment", TEST_COMMENT + CR, layout
+        assertEquals("Wrong comment", TEST_COMMENT + CRNORM, layout
                 .getCanonicalComment(TEST_KEY, false));
         assertEquals("Wrong property value", TEST_VALUE, config
                 .getString(TEST_KEY));
@@ -176,7 +180,7 @@ public class TestPropertiesConfigurationLayout extends TestCase
         builder.addProperty(TEST_KEY, TEST_VALUE + "2");
         layout.load(builder.getReader());
         assertEquals("Wrong combined comment",
-                TEST_COMMENT + CR + TEST_COMMENT, layout.getCanonicalComment(
+                TEST_COMMENT + CRNORM + TEST_COMMENT, layout.getCanonicalComment(
                         TEST_KEY, false));
         assertEquals("Wrong combined blanc numbers", 0, layout
                 .getBlancLinesBefore(TEST_KEY));
@@ -208,7 +212,7 @@ public class TestPropertiesConfigurationLayout extends TestCase
         builder.addComment(null);
         builder.addProperty(TEST_KEY, TEST_VALUE);
         layout.load(builder.getReader());
-        assertEquals("Wrong header comment", TEST_COMMENT + CR + CR
+        assertEquals("Wrong header comment", TEST_COMMENT + CRNORM + CRNORM
                 + TEST_COMMENT, layout.getCanonicalHeaderComment(false));
         assertNull("Wrong comment for property", layout.getComment(TEST_KEY));
     }
@@ -227,7 +231,7 @@ public class TestPropertiesConfigurationLayout extends TestCase
         builder.addComment(TEST_COMMENT);
         builder.addProperty(TEST_KEY, TEST_VALUE);
         layout.load(builder.getReader());
-        assertEquals("Wrong header comment", TEST_COMMENT + CR + CR
+        assertEquals("Wrong header comment", TEST_COMMENT + CRNORM + CRNORM
                 + TEST_COMMENT, layout.getCanonicalHeaderComment(false));
         assertEquals("Wrong comment for property", TEST_COMMENT, layout
                 .getCanonicalComment(TEST_KEY, false));
@@ -257,6 +261,7 @@ public class TestPropertiesConfigurationLayout extends TestCase
         assertEquals("Blanc lines before new property", 0, layout
                 .getBlancLinesBefore(TEST_KEY));
         assertTrue("No single line property", layout.isSingleLine(TEST_KEY));
+        assertEquals("Wrong separator", " = ", layout.getSeparator(TEST_KEY));
     }
 
     /**
@@ -402,9 +407,9 @@ public class TestPropertiesConfigurationLayout extends TestCase
                 .getCanonicalHeaderComment(false));
         assertFalse("Include property was stored", layout.getKeys().contains(
                 PropertiesConfiguration.getInclude()));
-        assertEquals("Wrong comment for property", TEST_COMMENT + CR
-                + "A nested header comment." + CR + "With multiple lines" + CR
-                + CR + "Second comment", layout.getCanonicalComment(TEST_KEY,
+        assertEquals("Wrong comment for property", TEST_COMMENT + CRNORM
+                + "A nested header comment." + CRNORM + "With multiple lines" + CRNORM
+                + CRNORM + "Second comment", layout.getCanonicalComment(TEST_KEY,
                 false));
     }
 
@@ -440,7 +445,7 @@ public class TestPropertiesConfigurationLayout extends TestCase
         layout.setComment("AnotherProperty", "AnotherComment");
         layout.setBlancLinesBefore("AnotherProperty", 2);
         layout.setSingleLine("AnotherProperty", true);
-        layout.setHeaderComment("A header comment" + CR + "for my properties");
+        layout.setHeaderComment("A header comment" + CRNORM + "for my properties");
         checkLayoutString("# A header comment" + CR + "# for my properties"
                 + CR + CR + "# " + TEST_COMMENT + CR + TEST_KEY + " = "
                 + TEST_VALUE + CR + TEST_KEY + " = " + TEST_VALUE + "2" + CR
@@ -578,7 +583,7 @@ public class TestPropertiesConfigurationLayout extends TestCase
     }
 
     /**
-     * Tests if the copy and the original are independend from each other.
+     * Tests if the copy and the original are independent from each other.
      */
     public void testInitCopyModify()
     {
@@ -596,6 +601,62 @@ public class TestPropertiesConfigurationLayout extends TestCase
                 .getBlancLinesBefore(TEST_KEY));
     }
 
+    /**
+     * Tests changing the separator for a property.
+     */
+    public void testSetSeparator() throws ConfigurationException
+    {
+        config.addProperty(TEST_KEY, TEST_VALUE);
+        layout.setSeparator(TEST_KEY, ":");
+        checkLayoutString(TEST_KEY + ":" + TEST_VALUE + CR);
+    }
+
+    /**
+     * Tests setting the global separator. This separator should override the
+     * separators for all properties.
+     */
+    public void testSetGlobalSeparator() throws ConfigurationException
+    {
+        final String sep = "=";
+        config.addProperty(TEST_KEY, TEST_VALUE);
+        config.addProperty("key2", "value2");
+        layout.setSeparator(TEST_KEY, " : ");
+        layout.setGlobalSeparator(sep);
+        checkLayoutString(TEST_KEY + sep + TEST_VALUE + CR + "key2" + sep
+                + "value2" + CR);
+    }
+
+    /**
+     * Tests setting the line separator.
+     */
+    public void testSetLineSeparator() throws ConfigurationException
+    {
+        final String lf = CR + CR;
+        config.addProperty(TEST_KEY, TEST_VALUE);
+        layout.setBlancLinesBefore(TEST_KEY, 2);
+        layout.setComment(TEST_KEY, TEST_COMMENT);
+        layout.setHeaderComment("Header comment");
+        layout.setLineSeparator(lf);
+        checkLayoutString("# Header comment" + lf + lf + lf + lf + "# "
+                + TEST_COMMENT + lf + TEST_KEY + " = " + TEST_VALUE + lf);
+    }
+
+    /**
+     * Tests whether the line separator is also taken into account within
+     * comments.
+     */
+    public void testSetLineSeparatorInComments() throws ConfigurationException
+    {
+        final String lf = "<-\n";
+        config.addProperty(TEST_KEY, TEST_VALUE);
+        layout.setComment(TEST_KEY, TEST_COMMENT + "\nMore comment");
+        layout.setHeaderComment("Header\ncomment");
+        layout.setLineSeparator(lf);
+        checkLayoutString("# Header" + lf + "# comment" + lf + lf + "# "
+                + TEST_COMMENT + lf + "# More comment" + lf + TEST_KEY + " = "
+                + TEST_VALUE + lf);
+    }
+
     /**
      * Helper method for filling the layout object with some properties.
      */
@@ -684,6 +745,7 @@ public class TestPropertiesConfigurationLayout extends TestCase
                     buf.append("! ");
                 }
                 buf.append(s);
+                commentCounter++;
             }
             buf.append(CR);
         }
