@@ -22,15 +22,15 @@ import java.math.BigDecimal;
 import java.util.Iterator;
 import java.util.List;
 
-import org.apache.commons.lang.SystemUtils;
-
 import junit.framework.TestCase;
 
+import org.apache.commons.lang.SystemUtils;
+
 /**
  * Test class for PropertyConverter.
  *
  * @author Emmanuel Bourg
- * @version $Revision: 662269 $, $Date: 2008-06-01 21:30:02 +0200 (So, 01 Jun 2008) $
+ * @version $Revision: 1034102 $, $Date: 2010-11-11 21:28:05 +0100 (Do, 11. Nov 2010) $
  */
 public class TestPropertyConverter extends TestCase
 {
@@ -115,6 +115,16 @@ public class TestPropertyConverter extends TestCase
                         .escapeDelimiters("C:\\Temp\\,D:\\Data\\", ','));
     }
 
+    /**
+     * Tests whether only the list delimiter can be escaped.
+     */
+    public void testEscapeListDelimiter()
+    {
+        assertEquals("Wrong escaped list delimiter", "C:\\Temp\\\\,D:\\Data\\",
+                PropertyConverter.escapeListDelimiter("C:\\Temp\\,D:\\Data\\",
+                        ','));
+    }
+
     public void testToIterator()
     {
         int[] array = new int[]{1, 2, 3};
@@ -354,7 +364,7 @@ public class TestPropertyConverter extends TestCase
         {
             return;
         }
-        
+
         try
         {
             assertEquals(enumObject, PropertyConverter.toEnum(new Integer(-1), enumClass));
