@@ -38,6 +38,7 @@ import junit.framework.TestCase;
 
 import org.apache.commons.configuration.reloading.FileAlwaysReloadingStrategy;
 import org.apache.commons.configuration.reloading.InvariantReloadingStrategy;
+import org.apache.commons.configuration.resolver.CatalogResolver;
 import org.apache.commons.configuration.tree.ConfigurationNode;
 import org.apache.commons.configuration.tree.xpath.XPathExpressionEngine;
 import org.xml.sax.SAXException;
@@ -47,10 +48,11 @@ import org.xml.sax.helpers.DefaultHandler;
 /**
  * test for loading and saving xml properties files
  *
- * @version $Id: TestXMLConfiguration.java 721895 2008-11-30 21:08:42Z oheger $
+ * @version $Id: TestXMLConfiguration.java 1095498 2011-04-20 20:07:31Z oheger $
  */
 public class TestXMLConfiguration extends TestCase
 {
+    private static final String CATALOG_FILES = "conf/catalog.xml";
     /** Constant for the used encoding.*/
     static final String ENCODING = "ISO-8859-1";
 
@@ -74,6 +76,14 @@ public class TestXMLConfiguration extends TestCase
     private String testProperties2 = new File("conf/testDigesterConfigurationInclude1.xml").getAbsolutePath();
     private String testBasePath = new File("conf").getAbsolutePath();
     private File testSaveConf = new File("target/testsave.xml");
+    private File testSaveFile = new File("target/testsample2.xml");
+    private String testFile2 = new File("conf/sample.xml").getAbsolutePath();
+
+    /** Constant for the number of test threads. */
+    private static final int THREAD_COUNT = 5;
+
+    /** Constant for the number of loops in tests with multiple threads. */
+    private static final int LOOP_COUNT = 100;
 
     private XMLConfiguration conf;
 
@@ -398,7 +408,7 @@ public class TestXMLConfiguration extends TestCase
      */
     public void testLoadFromURL() throws Exception
     {
-        URL url = new File(testProperties).toURL();
+        URL url = new File(testProperties).toURI().toURL();
         conf = new XMLConfiguration(url);
         assertEquals("value", conf.getProperty("element"));
         assertEquals(url, conf.getURL());
@@ -495,7 +505,7 @@ public class TestXMLConfiguration extends TestCase
      */
     public void testSaveToURL() throws Exception
     {
-        conf.save(testSaveConf.toURL());
+        conf.save(testSaveConf.toURI().toURL());
         XMLConfiguration checkConfig = new XMLConfiguration();
         checkConfig.setFile(testSaveConf);
         checkSavedConfig(checkConfig);
@@ -635,6 +645,68 @@ public class TestXMLConfiguration extends TestCase
         }
     }
 
+    public void testReloadingOOM() throws Exception
+    {
+        assertNotNull(conf.getReloadingStrategy());
+        assertTrue(conf.getReloadingStrategy() instanceof InvariantReloadingStrategy);
+        PrintWriter out = null;
+
+        try
+        {
+            out = new PrintWriter(new FileWriter(testSaveConf));
+            out.println("<?xml version=\"1.0\"?><config><test>1</test></config>");
+            out.close();
+            out = null;
+            conf.setFile(testSaveConf);
+            FileAlwaysReloadingStrategy strategy = new FileAlwaysReloadingStrategy();
+            strategy.setRefreshDelay(100);
+            conf.setReloadingStrategy(strategy);
+            conf.load();
+            assertEquals(1, conf.getInt("test"));
+
+            for (int i = 1; i < LOOP_COUNT; ++i)
+            {
+               assertEquals(1, conf.getInt("test"));
+            }
+        }
+        finally
+        {
+            if (out != null)
+            {
+                out.close();
+            }
+        }
+    }
+
+    /**
+     * Tests the refresh() method.
+     */
+    public void testRefresh() throws ConfigurationException
+    {
+        conf.setProperty("element", "anotherValue");
+        conf.refresh();
+        assertEquals("Wrong property after refresh", "value",
+                conf.getString("element"));
+    }
+
+    /**
+     * Tries to call refresh() when the configuration is not associated with a
+     * file.
+     */
+    public void testRefreshNoFile() throws ConfigurationException
+    {
+        conf = new XMLConfiguration();
+        try
+        {
+            conf.refresh();
+            fail("Could refresh a configuration without a file!");
+        }
+        catch (ConfigurationException cex)
+        {
+            // ok
+        }
+    }
+
     /**
      * Tests access to tag names with delimiter characters.
      */
@@ -765,6 +837,79 @@ public class TestXMLConfiguration extends TestCase
         assertEquals("a,b,c", conf2.getString("split.list1"));
         assertEquals(0, conf2.getMaxIndex("split.list1"));
         assertEquals("a\\,b\\,c", conf2.getString("split.list2"));
+        conf2 = new XMLConfiguration();
+        conf2.setExpressionEngine(new XPathExpressionEngine());
+        conf2.setDelimiterParsingDisabled(true);
+        conf2.setFile(new File(testProperties));
+        conf2.load();
+
+        assertEquals("a,b,c", conf2.getString("split/list3/@values"));
+        assertEquals(0, conf2.getMaxIndex("split/list3/@values"));
+        assertEquals("a\\,b\\,c", conf2.getString("split/list4/@values"));
+        assertEquals("a,b,c", conf2.getString("split/list1"));
+        assertEquals(0, conf2.getMaxIndex("split/list1"));
+        assertEquals("a\\,b\\,c", conf2.getString("split/list2"));
+    }
+
+     /**
+     * Tests string properties with list delimiters when delimiter parsing
+     * is disabled
+     */
+    public void testSaveWithDelimiterParsingDisabled() throws ConfigurationException {
+        XMLConfiguration conf = new XMLConfiguration();
+        conf.setExpressionEngine(new XPathExpressionEngine());
+        conf.setDelimiterParsingDisabled(true);
+        conf.setAttributeSplittingDisabled(true);
+        conf.setFile(new File(testProperties));
+        conf.load();
+
+        assertEquals("a,b,c", conf.getString("split/list3/@values"));
+        assertEquals(0, conf.getMaxIndex("split/list3/@values"));
+        assertEquals("a\\,b\\,c", conf.getString("split/list4/@values"));
+        assertEquals("a,b,c", conf.getString("split/list1"));
+        assertEquals(0, conf.getMaxIndex("split/list1"));
+        assertEquals("a\\,b\\,c", conf.getString("split/list2"));
+        // save the configuration
+        conf.save(testSaveConf.getAbsolutePath());
+
+        // read the configuration and compare the properties
+        XMLConfiguration checkConfig = new XMLConfiguration();
+        checkConfig.setFileName(testSaveConf.getAbsolutePath());
+        checkSavedConfig(checkConfig);
+        XMLConfiguration config = new XMLConfiguration();
+        config.setFileName(testFile2);
+        //config.setExpressionEngine(new XPathExpressionEngine());
+        config.setDelimiterParsingDisabled(true);
+        config.setAttributeSplittingDisabled(true);
+        config.load();
+        config.setProperty("Employee[@attr1]", "3,2,1");
+        assertEquals("3,2,1", config.getString("Employee[@attr1]"));
+        config.save(testSaveFile.getAbsolutePath());
+        config = new XMLConfiguration();
+        config.setFileName(testSaveFile.getAbsolutePath());
+        //config.setExpressionEngine(new XPathExpressionEngine());
+        config.setDelimiterParsingDisabled(true);
+        config.setAttributeSplittingDisabled(true);
+        config.load();
+        config.setProperty("Employee[@attr1]", "1,2,3");
+        assertEquals("1,2,3", config.getString("Employee[@attr1]"));
+        config.setProperty("Employee[@attr2]", "one, two, three");
+        assertEquals("one, two, three", config.getString("Employee[@attr2]"));
+        config.setProperty("Employee.text", "a,b,d");
+        assertEquals("a,b,d", config.getString("Employee.text"));
+        config.setProperty("Employee.Salary", "100,000");
+        assertEquals("100,000", config.getString("Employee.Salary"));
+        config.save(testSaveFile.getAbsolutePath());
+        checkConfig = new XMLConfiguration();
+        checkConfig.setFileName(testSaveFile.getAbsolutePath());
+        checkConfig.setExpressionEngine(new XPathExpressionEngine());
+        checkConfig.setDelimiterParsingDisabled(true);
+        checkConfig.setAttributeSplittingDisabled(true);
+        checkConfig.load();
+        assertEquals("1,2,3", checkConfig.getString("Employee/@attr1"));
+        assertEquals("one, two, three", checkConfig.getString("Employee/@attr2"));
+        assertEquals("a,b,d", checkConfig.getString("Employee/text"));
+        assertEquals("100,000", checkConfig.getString("Employee/Salary"));
     }
 
     /**
@@ -820,6 +965,18 @@ public class TestXMLConfiguration extends TestCase
         assertEquals("", conf.getProperty("empty2"));
     }
 
+    /**
+     * Tests the isEmpty() method for an empty configuration that was reloaded.
+     */
+    public void testEmptyReload() throws ConfigurationException
+    {
+        XMLConfiguration config = new XMLConfiguration();
+        assertTrue("Newly created configuration not empty", config.isEmpty());
+        config.save(testSaveConf);
+        config.load(testSaveConf);
+        assertTrue("Reloaded configuration not empty", config.isEmpty());
+    }
+
     /**
      * Tests whether the encoding is correctly detected by the XML parser. This
      * is done by loading an XML file with the encoding "UTF-16". If this
@@ -1000,14 +1157,14 @@ public class TestXMLConfiguration extends TestCase
      */
     public void testInitCopy() throws ConfigurationException
     {
-    	XMLConfiguration copy = new XMLConfiguration(conf);
+        XMLConfiguration copy = new XMLConfiguration(conf);
         assertEquals("value", copy.getProperty("element"));
         assertNull("Document was copied, too", copy.getDocument());
         ConfigurationNode root = copy.getRootNode();
         for(Iterator it = root.getChildren().iterator(); it.hasNext();)
         {
-        	ConfigurationNode node = (ConfigurationNode) it.next();
-        	assertNull("Reference was not cleared", node.getReference());
+            ConfigurationNode node = (ConfigurationNode) it.next();
+            assertNull("Reference was not cleared", node.getReference());
         }
 
         removeTestFile();
@@ -1307,7 +1464,7 @@ public class TestXMLConfiguration extends TestCase
         conf.save(testSaveConf);
         XMLConfiguration checkConfig = new XMLConfiguration();
         checkConfig.setFile(testSaveConf);
-        checkConfig.registerEntityId(publicId, dtdFile.toURL());
+        checkConfig.registerEntityId(publicId, dtdFile.toURI().toURL());
         checkConfig.setValidating(true);
         checkSavedConfig(checkConfig);
     }
@@ -1475,6 +1632,131 @@ public class TestXMLConfiguration extends TestCase
                 .getString("expressions[@value2]"));
     }
 
+    /**
+     * Tests modifying an XML document and saving it with schema validation enabled.
+     */
+    public void testSaveWithValidation() throws Exception
+    {
+        CatalogResolver resolver = new CatalogResolver();
+        resolver.setCatalogFiles(CATALOG_FILES);
+        conf = new XMLConfiguration();
+        conf.setEntityResolver(resolver);
+        conf.setFileName(testFile2);
+        conf.setSchemaValidation(true);
+        conf.load();
+        conf.setProperty("Employee.SSN", "123456789");
+        conf.validate();
+        conf.save(testSaveConf);
+        conf = new XMLConfiguration(testSaveConf);
+        assertEquals("123456789", conf.getString("Employee.SSN"));
+    }
+
+    /**
+     * Tests modifying an XML document and validating it against the schema.
+     */
+    public void testSaveWithValidationFailure() throws Exception
+    {
+        CatalogResolver resolver = new CatalogResolver();
+        resolver.setCatalogFiles(CATALOG_FILES);
+        conf = new XMLConfiguration();
+        conf.setEntityResolver(resolver);
+        conf.setFileName(testFile2);
+        conf.setSchemaValidation(true);
+        conf.load();
+        conf.setProperty("Employee.Email", "JohnDoe@apache.org");
+        try
+        {
+            conf.validate();
+            fail("No validation failure on save");
+        }
+        catch (Exception e)
+        {
+            Throwable cause = e.getCause();
+            assertNotNull("No cause for exception on save", cause);
+            assertTrue("Incorrect exception on save", cause instanceof SAXParseException);
+        }
+    }
+
+    public void testConcurrentGetAndReload() throws Exception
+    {
+        //final FileConfiguration config = new PropertiesConfiguration("test.properties");
+        final FileConfiguration config = new XMLConfiguration("test.xml");
+        config.setReloadingStrategy(new FileAlwaysReloadingStrategy());
+
+        assertTrue("Property not found", config.getProperty("test.short") != null);
+
+        Thread testThreads[] = new Thread[THREAD_COUNT];
+
+        for (int i = 0; i < testThreads.length; ++i)
+        {
+            testThreads[i] = new ReloadThread(config);
+            testThreads[i].start();
+        }
+
+        for (int i = 0; i < LOOP_COUNT; i++)
+        {
+            assertTrue("Property not found", config.getProperty("test.short") != null);
+        }
+
+        for (int i = 0; i < testThreads.length; ++i)
+        {
+            testThreads[i].join();
+        }
+    }
+
+    /**
+     * Tests whether a windows path can be saved correctly. This test is related
+     * to CONFIGURATION-428.
+     */
+    public void testSaveWindowsPath() throws ConfigurationException
+    {
+        conf.clear();
+        conf.addProperty("path", "C:\\Temp");
+        StringWriter writer = new StringWriter();
+        conf.save(writer);
+        String content = writer.toString();
+        assertTrue("Path not found: " + content,
+                content.indexOf("<path>C:\\Temp</path>") >= 0);
+        conf.save(testSaveFile);
+        XMLConfiguration conf2 = new XMLConfiguration(testSaveFile);
+        assertEquals("Wrong windows path", "C:\\Temp",
+                conf2.getString("path"));
+    }
+
+    private class ReloadThread extends Thread
+    {
+        FileConfiguration config;
+
+        ReloadThread(FileConfiguration config)
+        {
+            this.config = config;
+        }
+        public void run()
+        {
+            for (int i = 0; i < LOOP_COUNT; i++)
+            {
+                config.reload();
+            }
+        }
+    }
+
+    /**
+     * Tests whether an attribute can be set to an empty string. This test is
+     * related to CONFIGURATION-446.
+     */
+    public void testEmptyAttribute() throws ConfigurationException
+    {
+        String key = "element3[@value]";
+        conf.setProperty(key, "");
+        assertTrue("Key not found", conf.containsKey(key));
+        assertEquals("Wrong value", "", conf.getString(key));
+        conf.save(testSaveConf);
+        conf = new XMLConfiguration();
+        conf.load(testSaveConf);
+        assertTrue("Key not found after save", conf.containsKey(key));
+        assertEquals("Wrong value after save", "", conf.getString(key));
+    }
+
     /**
      * Prepares a configuration object for testing a reload operation.
      *
