@@ -36,7 +36,7 @@ import org.apache.commons.configuration.StrictConfigurationComparator;
 
 /**
  * @author Emmanuel Bourg
- * @version $Revision: 590474 $, $Date: 2007-10-30 22:35:11 +0100 (Di, 30 Okt 2007) $
+ * @version $Revision: 952624 $, $Date: 2010-06-08 14:00:58 +0200 (Di, 08. Jun 2010) $
  */
 public class TestPropertyListConfiguration extends TestCase
 {
@@ -252,6 +252,29 @@ public class TestPropertyListConfiguration extends TestCase
         }
     }
 
+    public void testSaveEmptyDictionary() throws Exception
+    {
+        File savedFile = new File("target/testsave.plist");
+
+        // remove the file previously saved if necessary
+        if (savedFile.exists())
+        {
+            assertTrue(savedFile.delete());
+        }
+
+        // save the configuration
+        String filename = savedFile.getAbsolutePath();
+        config.save(filename);
+
+        assertTrue("The saved file doesn't exist", savedFile.exists());
+
+        // read the configuration and compare the properties
+        PropertyListConfiguration checkConfig = new PropertyListConfiguration(new File(filename));
+
+        assertFalse(config.getRootNode().getChildren("empty-dictionary").isEmpty());
+        assertFalse(checkConfig.getRootNode().getChildren("empty-dictionary").isEmpty());
+    }
+
     public void testQuoteString()
     {
         assertEquals("null string", null, config.quoteString(null));
@@ -300,8 +323,8 @@ public class TestPropertyListConfiguration extends TestCase
 
     public void testInitCopy()
     {
-    	PropertyListConfiguration copy = new PropertyListConfiguration(config);
-    	assertFalse("Nothing was copied", copy.isEmpty());
+        PropertyListConfiguration copy = new PropertyListConfiguration(config);
+        assertFalse("Nothing was copied", copy.isEmpty());
     }
 
     /**
